
__all__ = ['TUPLE', 'LIST', 'DICT']

from .base import Head, heads_precedence

def init_module(m):
    from .base import heads
    for n,h in heads.iterNameValue(): setattr(m, n, h)

class ContainerHead(Head):

    def data_to_str_and_precedence(self, cls, seq):
        c_p = heads_precedence.COMMA
        s_p = getattr(heads_precedence, repr(self))
        l = []
        for item in seq:
            i, i_p = item.head.data_to_str_and_precedence(cls, item.data)
            if i_p < c_p: i = '('+i+')'
            l.append(i)
        s1, s2 = self.parenthesis
        if self is TUPLE and len(l)==1:
            return s1 + l[0] + ',' + s2, s_p
        return s1 + ', '.join(l) + s2, s_p

class TupleHead(ContainerHead):
    """
    TupleHead represents n-tuple,
    data is n-tuple of expressions.
    """
    parenthesis = '()'

    def __repr__(self): return 'TUPLE'

class ListHead(ContainerHead):
    """
    ListHead represents n-list,
    data is n-tuple of expressions.
    """
    parenthesis = '[]'    
    def __repr__(self): return 'LIST'

class DictHead(ContainerHead):
    """
    DictHead represents n-dict,
    data is n-tuple of expression pairs.
    """

    def data_to_str_and_precedence(self, cls, seq2):
        c_p = heads_precedence.COMMA
        colon_p = heads_precedence.COLON
        s_p = getattr(heads_precedence, repr(self))
        l = []
        for key, value in seq2:
            k, k_p = key.head.data_to_str_and_precedence(cls, key.data)
            if k_p < colon_p: k = '('+k+')'
            v, v_p = value.head.data_to_str_and_precedence(cls, value.data)
            if v_p < colon_p: v = '('+v+')'
            l.append(k + ':' + v)
        return '{' + ', '.join(l) + '}', heads_precedence.DICT

    def __repr__(self): return 'DICT'

TUPLE = TupleHead()
LIST = ListHead()
DICT = DictHead()



"""
FRED API documentation: http://api.stlouisfed.org/docs/fred/

Core functionality for interacting with the FRED API.
"""

import os
from itertools import ifilter

import requests
from relaxml import xml


class Fred(object):
    """An easy-to-use Python wrapper over the St. Louis FRED API."""

    def __init__(self, api_key='', xml_output=False):
        if 'FRED_API_KEY' in os.environ:
            self.api_key = os.environ['FRED_API_KEY']
        else:
            self.api_key = api_key
        self.xml = xml_output
        self.endpoint = 'http://api.stlouisfed.org/fred/'

    def _create_path(self, *args):
        """Create the URL path with the Fred endpoint and given arguments."""
        args = ifilter(None, args)
        path = self.endpoint + '/'.join(args)
        return path

    def get(self, *args, **kwargs):
        """Perform a GET request againt the Fred API endpoint."""
        location = args[0]
        params = self._get_keywords(location, kwargs)
        url = self._create_path(*args)
        request = requests.get(url, params=params)
        content = request.content
        self._request = request
        return self._output(content)

    def _get_keywords(self, location, keywords):
        """Format GET request's parameters from keywords."""
        if 'xml' in keywords:
            keywords.pop('xml')
            self.xml = True
        if 'id' in keywords:
            if location != 'series':
                location = location.rstrip('s')
            key = '%s_id' % location
            value = keywords.pop('id')
            keywords[key] = value
        if 'start' in keywords:
            time = keywords.pop('start')
            keywords['realtime_start'] = time
        if 'end' in keywords:
            time = keywords.pop('end')
            keywords['realtime_end'] = time
        if 'sort' in keywords:
            order = keywords.pop('sort')
            keywords['sort_order'] = order
        keywords['api_key'] = self.api_key
        return keywords

    def _output(self, content):
        """Return the output from a given GET request."""
        if self.xml:
            return content
        return xml(content)

    def category(self, path=None, **kwargs):
        """
        Get a specific category.

        >>> Fred().category(category_id=125)
        """
        return self.get('category', path, **kwargs)

    def release(self, path=None, **kwargs):
        """
        Get a release of economic data.

        >>> Fred().release('series', release_id=51)
        """
        return self.get('release', path, **kwargs)

    def releases(self, path=None, **kwargs):
        """
        Get all releases of economic data.

        >>> Fred().releases('dates', limit=10)
        """
        return self.get('releases', path, **kwargs)

    def series(self, path=None, **kwargs):
        """
        Get economic series of data.

        >>> Fred().series('search', search_text="money stock")
        """
        return self.get('series', path, **kwargs)

    def source(self, path=None, **kwargs):
        """
        Get a single source of economic data.

        >>> Fred().source(source_id=51)
        """
        return self.get('source', path, **kwargs)

    def sources(self, path=None, **kwargs):
        """
        Get all of FRED's sources of economic data.

        >>> Fred().sources()
        """
        return self.get('sources', path, **kwargs)




#!/usr/bin/env python
from numpy import abs, array, average, corrcoef, mat, shape, std, sum, transpose, zeros
from numpy.linalg import svd

import_ok = False
     
__author__ = "Henning Risvik"
__date__ = "February 2008"
__version__ = "1.1.02"
    
################### Preprocessing Methods ###################
def mean_center(X):
    """
    
    @param X: 2-dimensional matrix of number data 
    @type X: numpy array
    
    
    @return: Mean centered X (always has same dimensions as X)
    
    """
    (rows, cols) = shape(X)
    new_X = zeros((rows, cols), float)
    _averages = average(X, 0)
        
    for row in range(rows):
        new_X[row, 0:cols] = X[row, 0:cols] - _averages[0:cols]
    return new_X
        
        
def standardization(X):        
    """
    
    @param X: 2-dimensional matrix of number data 
    @type X: numpy array
    
    
    @return: Standardized X (always has same dimensions as X)
    
    """
    (rows, cols) = shape(X)
    new_X = zeros((rows, cols))
    _STDs = std(X, 0)
        
    for value in _STDs:
        if value == 0: raise ZeroDivisionError, 'division by zero, cannot proceed'
        
    for row in range(rows):
        new_X[row, 0:cols] = X[row, 0:cols] / _STDs[0:cols]
    return new_X        


       
################### NIPALS array help functions ###################
def get_column(E):
    """
    Get an acceptable column-vector of E.
    
    @param E: 2-dimensional matrix of number data 
    @type E: numpy array    
    
    @return: a non-zero vector
    """
    (rows, cols) = shape(E)
    for col_ind in range(cols):
        t = E[:,col_ind] # extract a column
        if (vec_inner(t) > 0):
            return t
    raise ValueError, 'all column vectors in E are zero vectors' # error: sum of matrix is 0


def get_column_mat(E):
    """
    NIPALS matrix help function.
    Get an acceptable column-vector of E.
    
    @param E: 2-dimensional matrix of number data 
    @type E: numpy matrix    
    
    @return: a non-zero vector
    """
    (rows, cols) = shape(E)
    for col_ind in range(cols):
        t = E[:,col_ind] # extract a column
        eig = transpose(t)*t
        if (eig > 0):
            return t
    raise ValueError, 'all column vectors in E are zero vectors' # error: sum of matrix is 0

def vec_inner(v):
    """
    @param v: Vector of number data.
    @type v: numpy array     
    
    @return: transpose(v) * v (float or int)
    """
    return sum(v * v);

def mat_prod(A, x):
    """    
    @param A: 2-dimensional matrix of number data.
    @type A: numpy array 
    
    @param x: Vector of number data.
    @type x: numpy array         

    @return: b of (Ax = b). Product of:  matrix A (m,n) * vector x (n) = vector b (m)
    """
    #m = A.shape[0]
    #b = zeros((m), float)
    
    # calc: Ax = b
    #for i in range(m):
    #    b[i] = sum(A[i,:]*x)

    return array(map(lambda a: sum(a[:]*x), A))

def remove_tp_prod(E, t, p):
    """
    
    sets: E = E - (t*transpose(p))   
    E: (m, n)-matrix, (t*transpose(p)): (m, n)-matrix   
    
    
    @param E: 2-dimensional matrix of number data.
    @type E: numpy array 
    
    @param t: Vector of number data. Current Scores (of PC_i).
    @type t: numpy array         

    @param p: Vector of number data. Current Loading (of PC_i).
    @type p: numpy array   


    @return: None   


    """
    
    m = E.shape[0]
    for i in range(m):
        E[i, :] = E[i, :] - (t[i] * p)
       

################### NIPALS Algorithm ###################
"""
  Estimation of PC components with the iterative NIPALS method: 


  E[0] = mean_center(X)  (the E-matrix for the zero-th PC)

  t = E(:, 0)  (a column in X (mean centered) is set as starting t vector)

  for i=1 to (PCs):
    
    1  p=(E[i-1]'t) / (t't)  Project X onto t to find the corresponding (improve estimated) loading p
    
    2  p = p * (p'p)^-0.5  Normalise loading vector p to length 1
    
    3  t = (E[i-1]p) / (p'p)  Project X onto p to find corresponding (improve estimated) score vector t
    
    4  Check for convergence, if difference between eigenval_new and eigenval_old is larger than threshold*eigenval_new return to step 1
    
    5  E[i] = E[i-1] - tp'  Remove the estimated PC component from E[i-1] 
    
"""    
def nipals_mat(X, PCs, threshold, E_matrices):
    """
    
    
    PCA by NIPALS using numpy matrix
    
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param PCs: Number of Principal Components.
    @type PCs: int
    
    @param threshold: Convergence check value. For checking on convergence to zero difference (e.g. 0.000001). 
    @type threshold: float
    
    @param E_matrices: If E-matrices should be retrieved or not. E-matrices (for each PC) or explained_var (explained variance for each PC).
    @type E_matrices: bool
    
    @return: (Scores, Loadings, E)

    """
    (rows, cols) = shape(X)

    maxPCs = min(rows, cols) # max number of PCs is min(objects, variables)
    if maxPCs < PCs: PCs = maxPCs # change to maxPCs if PCs > maxPCs
    
    Scores = zeros((rows, PCs), float) # all Scores (T)
    Loadings = zeros((PCs, cols), float) # all Loadings (P)
    
    E = mat(X.copy()) #E[0]  (should already be mean centered)


    if E_matrices:
        Error_matrices = zeros((PCs, rows, cols), float) # all Error matrices (E)
    else:
        explained_var = zeros((PCs))
        tot_explained_var = 0
    
        # total object residual variance for PC[0] (calculating from E[0])
        e_tot0 = 0 # for E[0] the total object residual variance is 100%
        for k in range(rows):
            e_k = array(E[k, :])**2
            e_tot0 += sum(e_k)
            
    

    t = get_column_mat(E) # extract a column
    
   
    
    # do iterations (0, PCs)
    for i in range(PCs):
        convergence = False
        ready_for_compare = False
        E_t = transpose(E)
        
        while not convergence:
            eigenval = float(transpose(t)*t)
            p = (E_t*t) / eigenval # ........................................... step 1
            
            _p = float(transpose(p)*p)        
            p = p * _p**(-0.5) # ............................................... step 2
            t = (E*p) / (transpose(p)*p) # ..................................... step 3
            
            
            eigenval_new = float(transpose(t)*t)
            if not ready_for_compare:
                ready_for_compare = True
            else: # ready for convergence check
                if (eigenval_new - eigenval_old) < threshold*eigenval_new: # ... step 4
                    convergence = True           
            eigenval_old = eigenval_new;
            
            
        p = transpose(p)
        E = E - (t*p) # ........................................................ step 5
        
        # add Scores and Loadings for PC[i] to the collection of all PCs
        _t = array(t) # NOT optimal
        Scores[:, i] = _t[:,0]; Loadings[i, :] = p[0,:] # co
     
        if E_matrices:
		# complete error matrix
		# can calculate object residual variance (row-wise) or variable resiudal variance (column-wise)
		# total residual variance can also be calculated
		
		Error_matrices[i] = E.copy()
        
        else:
		# total object residual variance for E[i]
		e_tot = 0
		
		for k in range(rows):
		    e_ = zeros((cols), float)
		    for k_col in range(cols):
		        e_[k_col] = E[k, k_col]*E[k, k_col]
		    e_tot += sum(e_)
		tot_obj_residual_var =  (e_tot / e_tot0)
		explained_var[i] = 1 - tot_obj_residual_var - tot_explained_var
		tot_explained_var += explained_var[i]
		
		
		

    if E_matrices:
        return Scores, Loadings, Error_matrices
    return Scores, Loadings, explained_var    



def nipals_arr(X, PCs, threshold, E_matrices):
    """
    
    PCA by NIPALS using numpy array
    
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param PCs: Number of Principal Components.
    @type PCs: int
    
    @param threshold: Convergence check value. For checking on convergence to zero (e.g. 0.000001). 
    @type threshold: float
    
    @param E_matrices: If E-matrices should be retrieved or not. E-matrices (for each PC) or explained_var (explained variance for each PC).
    @type E_matrices: bool
    
    @return: (Scores, Loadings, E)


    """
    
    (rows, cols) = shape(X)
    maxPCs = min(rows, cols) # max number of PCs is min(objects, variables)
    if maxPCs < PCs: PCs = maxPCs # change to maxPCs if PCs > maxPCs

    Scores = zeros((rows, PCs), float) # all Scores (T)
    Loadings = zeros((PCs, cols), float) # all Loadings (P)
    
    E = X.copy() #E[0]  (should already be mean centered)
    
    if E_matrices:
        Error_matrices = zeros((PCs, rows, cols), float) # all Error matrices (E)
    else:
        explained_var = zeros((PCs))
        tot_explained_var = 0
    
        # total object residual variance for PC[0] (calculating from E[0])
        e_tot0 = 0 # for E[0] the total object residual variance is 100%
        for k in range(rows):
            e_k = E[k, :]**2
            e_tot0 += sum(e_k)      
            
    

    t = get_column(E) # extract a column
    p = zeros((cols), float)
    
    # do iterations (0, PCs)
    for i in range(PCs):
        convergence = False
        ready_for_compare = False
        E_t = transpose(E)
        
        while not convergence:
            _temp = vec_inner(t)
            p = mat_prod(E_t, t) / _temp # ..................................... step 1
            
            _temp = vec_inner(p)**(-0.5)
            p = p * _temp # .................................................... step 2
            
            _temp = vec_inner(p)
            t = mat_prod(E, p) / _temp # ....................................... step 3
            
            
            eigenval_new = vec_inner(t)
            if not ready_for_compare:
                ready_for_compare = True
            else: # ready for convergence check
                if (eigenval_new - eigenval_old) < threshold*eigenval_new: # ... step 4
                    convergence = True           
            eigenval_old = eigenval_new;

        remove_tp_prod(E, t, p) # .............................................. step 5
        
        # add Scores and Loadings for PC[i] to the collection of all PCs
        Scores[:, i] = t; Loadings[i, :] = p
        
        
        if E_matrices:
		# complete error matrix
		# can calculate object residual variance (row-wise) or variable resiudal variance (column-wise)
		# total residual variance can also be calculated
		
		Error_matrices[i] = E.copy()
        
        else:
		# total object residual variance for E[i]
		e_tot = 0
		for k in range(rows):
		    e_k = E[k, :]**2
		    e_tot += sum(e_k)
		tot_obj_residual_var =  (e_tot / e_tot0)
		explained_var[i] = 1 - tot_obj_residual_var - tot_explained_var
		tot_explained_var += explained_var[i]

    if E_matrices:
        return Scores, Loadings, Error_matrices
    else:
        return Scores, Loadings, explained_var 
    
    

def nipals_c(X, PCs, threshold, E_matrices):
    """  
    
    PCA by NIPALS using python c extension
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param PCs: Number of Principal Components.
    @type PCs: int
    
    @param threshold: Convergence check value. For checking on convergence to zero (e.g. 0.000001). 
    @type threshold: float
    
    @param E_matrices: If E-matrices should be retrieved or not. E-matrices (for each PC) or explained_var (explained variance for each PC).
    @type E_matrices: bool
    
    @return: (Scores, Loadings, E)
    

    """
    
    if not import_ok:
        raise ImportError, "could not import c_nipals python extension"
    else:  

        (rows, cols) = shape(X)

	maxPCs = min(rows, cols) # max number of PCs is min(objects, variables)
	if maxPCs < PCs: PCs = maxPCs # change to maxPCs if PCs > maxPCs

	Scores = zeros((rows, PCs), float) # all Scores (T)
	Loadings = zeros((PCs, cols), float) # all Loadings (P)

	E = X.copy() #E[0]  (should already be mean centered)

        if E_matrices:
            Error_matrices = zeros((PCs, rows, cols), float) # all Error matrices (E)
            c_nipals.nipals2(Scores, Loadings, E, Error_matrices, PCs, threshold)
            return Scores, Loadings, Error_matrices
        else:
	    explained_var = c_nipals.nipals(Scores, Loadings, E, PCs, threshold)
	    return Scores, Loadings, explained_var



################### Principal Component Analysis (using NIPALS) ###################
def PCA_nipals(X, standardize=True, PCs=10, threshold=0.0001, E_matrices=False):
    """
    
    PCA by NIPALS and get Scores, Loadings, E
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param standardize: Wheter X should be standardized or not.
    @type standardize: bool
    
    @param PCs: Number of Principal Components.
    @type PCs: int
    
    @param threshold: Convergence check value. For checking on convergence to zero (e.g. 0.000001). 
    @type threshold: float
    
    @param E_matrices: If E-matrices should be retrieved or not. E-matrices (for each PC) or explained_var (explained variance for each PC).
    @type E_matrices: bool
    
    @return: nipals_mat(X, PCs, threshold, E_matrices)

    """

    X = mean_center(X)
        
    if standardize:
        X = standardization(X)
    
    return nipals_mat(X, PCs, threshold, E_matrices) 
    

def PCA_nipals2(X, standardize=True, PCs=10, threshold=0.0001, E_matrices=False):
    """
    
    PCA by NIPALS and get Scores, Loadings, E
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param standardize: Wheter X should be standardized or not.
    @type standardize: bool
    
    @param PCs: Number of Principal Components.
    @type PCs: int
    
    @param threshold: Convergence check value. For checking on convergence to zero (e.g. 0.000001). 
    @type threshold: float
    
    @param E_matrices: If E-matrices should be retrieved or not. E-matrices (for each PC) or explained_var (explained variance for each PC).
    @type E_matrices: bool
    
    @return: nipals_arr(X, PCs, threshold, E_matrices)

    """

    X = mean_center(X)
        
    if standardize:
        X = standardization(X)
    
    return nipals_arr(X, PCs, threshold, E_matrices)     


def PCA_nipals_c(X, standardize=True, PCs=10, threshold=0.0001, E_matrices=False):
    """
    
    PCA by NIPALS and get Scores, Loadings, E
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param standardize: Wheter X should be standardized or not.
    @type standardize: bool
    
    @param PCs: Number of Principal Components.
    @type PCs: int
    
    @param threshold: Convergence check value. For checking on convergence to zero (e.g. 0.000001). 
    @type threshold: float
    
    @param E_matrices: If E-matrices should be retrieved or not. E-matrices (for each PC) or explained_var (explained variance for each PC).
    @type E_matrices: bool
    
    @return: nipals_c(X, PCs, threshold, E_matrices)

    """

    """ USING C PYTHON EXTENSION """
    X = mean_center(X)
        
    if standardize:
        X = standardization(X)
    
    return nipals_c(X, PCs, threshold, E_matrices)      
    
    
    
################### Principal Component Analysis (using SVD) ###################
def PCA_svd(X, standardize=True):
    """   
    PCA by SVD and get Scores, Loadings, E
    Remake of method made by Oliver Tomic Ph.D.
    
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param standardize: Wheter X should be standardized or not.
    @type standardize: bool
    
    @return: (Scores, Loadings, explained_var)

    """

    X = mean_center(X)
    #print X
        
    if standardize:
        X = standardization(X)    
        
    (rows, cols) = shape(X)
    
    # Singular Value Decomposition
    [U, S, V] = svd(X)
    
    # adjust if matrix shape does not match:
    if shape(S)[0] < shape(U)[0]: U = U[:, 0:shape(S)[0]]
    
    Scores = U * S # all Scores (T)
    Loadings = V # all Loadings (P)
    
    variances = S**2 / cols
    variances_sum = sum(variances)
    explained_var = variances / variances_sum
    
    return Scores, Loadings, explained_var



################### Correlation Loadings ###################
def CorrelationLoadings(X, Scores):
    """
    Get correlation loadings matrix based on Scores (T of PCA) and X (original variables, not mean centered).
    Remake of method made by Oliver Tomic Ph.D.
       
    @param X: 2-dimensional matrix of number data. 
    @type X: numpy array
    
    @param Scores: Scores of PCA (T).
    @type Scores: numpy array

    @return: Returns the correlation loadings matrix

    """
        
    # Creates empty matrix for correlation loadings
    PCs = shape(Scores)[1] # number of PCs
    rows = shape(X)[1] # number of objects (rows) in X
    CorrLoadings = zeros((PCs, rows), float)
        
    # Calculates correlation loadings with formula:
    # correlation = cov(x,y)/(std(x)*std(y))
        
    # For each PC in score matrix
    for i in range(PCs):
        Scores_PC_i = Scores[:, i] # Scores for PC[i]
            
        # For each variable/attribute in X 
        for row in range(rows):
            orig_vars = X[:, row] # column of variables in X
            corrs = corrcoef(Scores_PC_i, orig_vars)
            CorrLoadings[i, row] = corrs[0,1]
                
    return CorrLoadings        



#!/usr/bin/env python

def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    from numpy.distutils.system_info import get_info
    config = Configuration('pymaclab', parent_package, top_path)
    config.add_subpackage('filters')
    config.add_subpackage('dsge')
    config.add_subpackage('stats')
    config.add_subpackage('dattrans')
    config.add_subpackage('linalg')
    config.add_subpackage('tests')
    config.add_data_dir('modfiles')
    return config

if __name__ == '__main__':
    print('This is the wrong setup.py file to run')



# File setup.py
import platform
def configuration(parent_package='',top_path=None):
    from numpy.distutils.misc_util import Configuration
    from numpy.distutils.system_info import get_info
    config = Configuration('solvers',parent_package,top_path)
    lapack = dict(get_info('lapack_opt'))
    # Branch here for different operating systems
    if platform.system() == 'Linux':
        config.add_extension('isolab', sources=['src/isolab.pyf',
                                                'src/solab.f90',
                                                'src/isolab.f90'],
                                       libraries=['lapack'],
                                       library_dirs=lapack['library_dirs'])
    elif platform.system() == 'Darwin':
        lapack['library_dirs'] = ['/usr/lib']
        config.add_extension('isolab', sources=['src/isolab.pyf',
                                                'src/solab.f90',
                                                'src/isolab.f90'],
                                       libraries=['lapack'],
                                       library_dirs=lapack['library_dirs'])
    elif platform.system() == 'Windows':
        config.add_extension('isolab', sources=['src/isolab.pyf',
                                                'src/solab.f90',
                                                'src/isolab.f90'],
                                       libraries=['lapack'],
                                       library_dirs=lapack['library_dirs'])
    return config

if __name__ == "__main__":
    from numpy.distutils.core import setup
    setup(**configuration(top_path='').todict())



#
# Author: Pearu Peterson
# Created: February 2008
#
""" Provides PolynomialRing class.
"""

__docformat__ = "restructuredtext"
__all__ = "PolynomialRing"

from ..core import classes, IntegerList, Expr
from ..utils import SYMBOL, NUMBER, ADD, MUL, POW, SPARSE_POLY, SUB, DIV
from ..basealgebra.algebra import Algebra
from ..ring import CommutativeRing
from ..basealgebra.verbatim import Verbatim
from ..arithmetic.numbers import div
from ..arithmetic.number_theory import multinomial_coefficients

def cmp_symbols(x, y):
    return cmp(str(x), str(y))

class PolynomialRingFactory(type):
    """ Factory of polynomial rings with symbols and coefficient ring.
    """
    def __new__(typ, name, bases, attrdict):
        if not attrdict.has_key('ring'):
            attrdict['ring'] = classes.Calculus
        if not attrdict.has_key('variables'):
            attrdict['variables'] = ()
            attrdict['nvars'] = 0
        cls = type.__new__(typ, name, bases, attrdict)
        cls.zero = cls.Number(0)
        cls.one = cls.Number(1)
        return cls

    def __eq__(self, other):
        if isinstance(other, PolynomialRingFactory):
            return self.ring==other.ring and self.variables==other.variables
        return False

    def __ne__(self, other):
        return not self==other

    def __getitem__(self, ring_info, cache={}):
        """ Return a new polynomial ring class

        Examples::

          PolynomialRing[<seq of variables>, <coefficient ring>]
          PolynomialRing[<n>, <coefficient ring>] is
                PolynomialRing[['X%s'%i for i in range(<n>)], <coefficient ring>]
          PolynomialRing[<variable info>] is
                PolynomialRing[<variable info>, Calculus]
        """
        if isinstance(ring_info, (int, long)):
            nvars = ring_info
            assert nvars>=0,`nvars`
            variables = ['X%s'%i for i in range(nvars)]
            ring = classes.Calculus
        elif isinstance(ring_info, tuple) and isinstance(ring_info[-1], type):
            if len(ring_info)==2:
                var_info, ring = ring_info
                if isinstance(var_info,(int, long)):
                    nvars = var_info
                    assert nvars>=0,`nvars`
                    variables = ['X%s'%i for i in range(nvars)]
                elif isinstance(var_info,(tuple, list, set)):
                    variables = var_info
                elif isinstance(var_info, str):
                    variables = var_info,
                else:
                    raise TypeError(`ring_info`)
            else:
                variables = ring_info[:-1]
                ring = ring_info[-1]
        elif isinstance(ring_info, (tuple, list, set)):
            variables = ring_info
            ring = classes.Calculus
        elif isinstance(ring_info, str):
            variables = ring_info,
            ring = classes.Calculus        
        else:
            raise TypeError(`ring_info`)
        variables = tuple(sorted(variables, cmp=cmp_symbols))
        nvars = len(variables)

        r = None #cache.get((variables, ring))
        if r is None:
            name = '%s[%s, %s]' % (self.__name__, tuple(variables), ring.__name__)
            r = PolynomialRingFactory(name,
                                      (self,),
                                      dict(nvars=nvars, ring = ring,
                                           variables = variables))
            #cache[variables, ring] = r
        return r

    def is_subring(self, other):
        """ Check if self contains other as a subring, i.e. whether
        the other instances can be converted to self instances.
        """
        if not isinstance(other, PolynomialRingFactory):
            return False
        if self.ring != other.ring:
            return False
        return set(other.variables).issubset(self.variables)

AdditiveTuple = IntegerList

class PolynomialRing(CommutativeRing):
    """ Base class to polynomial rings that holds polynomial information
    using pairs ``(<exponents>: <coefficient>)`` stored in Python dictionary.

    Suitable for representing sparse multivariate polynomials.
    """

    __slots__ = ['_degree', '_ldegree']
    _degree = None
    _ldegree = None
    _str_value = None
    
    __metaclass__ = PolynomialRingFactory

    __str__ = CommutativeRing.__str__
    __repr__ = CommutativeRing.__repr__

    @classmethod
    def convert(cls, data, typeerror=True):
        if isinstance(data, list):
            if cls.nvars==1 and data:
                if type(data[0]) not in [tuple,list,set]:
                    data = [(i,c) for (i,c) in enumerate(data) if c]
            data = dict(data)
            return cls(SPARSE_POLY, data)
        elif isinstance(data, dict):
            return cls(SPARSE_POLY, data)
        r = super(CommutativeRing, cls).convert(data, typeerror=True)
        return r

    @classmethod
    def get_operand_algebra(cls, head, index=0):
        if head in [ADD, SUB, MUL]:
            return cls
        if head is POW:
            if index==0:
                return cls
            if index==1:
                return int
        return cls.handle_get_operand_algebra_failure(head, index)
    
    def __eq__(self, other):
        if type(other)==type(self):
            return self.pair == other.pair
        return CommutativeRing.__eq__(self, other)

    @classmethod
    def Symbol(cls, obj):
        """ Return symbol element of a polynomial ring. The result
        may be an instance of a super polynomial ring.

        Examples::

          r = PolynomialRing['x']
          r.Symbol('x') -> r({1:1})
          r.Symbol('y') -> PolynomialRing['x','y']({(0,1):1})
        """
        try:
            i = list(cls.variables).index(obj)
        except ValueError:
            i = None
        if i is None:
            cls = PolynomialRing[cls.variables+(obj,), cls.ring]
            i = list(cls.variables).index(obj)
        l = [0]*cls.nvars
        l[i] = 1
        return cls(SPARSE_POLY, {AdditiveTuple(l):1})

    @classmethod
    def convert_symbol(cls, obj):
        try:
            i = list(cls.variables).index(obj)
        except ValueError:
            i = None
        if i is None:
            cls = PolynomialRing[cls.variables+(obj,), cls.ring]
            i = list(cls.variables).index(obj)
        l = [0]*cls.nvars
        l[i] = 1
        return cls(SPARSE_POLY, {AdditiveTuple(l):1})

    @classmethod
    def Number(cls, obj):
        """ Return number element of a polynomial ring.

        Examples::

          r = PolynomialRing['x']
          r.Number(2) -> r({0:2})
        """
        data = {AdditiveTuple((0,)*cls.nvars): obj} if obj else {}        
        return cls(SPARSE_POLY, data)

    @classmethod
    def Add(cls, *seq):
        r = cls(SPARSE_POLY, {})
        for t in seq:
            tcls = t.__class__
            if cls==tcls:
                iadd_POLY_POLY(r, t, cls)
            elif cls.is_subring(tcls):
                t = t.as_algebra(cls)
                assert cls==t.__class__,`cls,t`
                iadd_POLY_POLY(r, t, cls)
            elif tcls.is_subring(cls):
                cls = tcls
                r = r.as_algebra(cls)
                iadd_POLY_POLY(r, t, cls)
            elif cls.ring==tcls.ring:
                cls = PolynomialRing[cls.variables+tcls.variables, cls.ring]
                r = r.as_algebra(cls)
                t = t.as_algebra(cls)
                iadd_POLY_POLY(r, t, cls)
            else:
                raise NotImplementedError(`r, t`)
        if not r.data:
            return cls.Number(0)
        return r

    @classmethod
    def Mul(cls, *seq):
        r = cls.one
        for t in seq:
            tcls = t.__class__
            if cls==tcls:
                r = mul_POLY_POLY(r, t, cls)
            elif cls.is_subring(tcls):
                t = t.as_algebra(cls)
                assert cls==t.__class__,`cls,t`
                r = mul_POLY_POLY(r, t, cls)
            elif tcls.is_subring(cls):
                cls = tcls
                r = r.as_algebra(cls)
                r = mul_POLY_POLY(r, t, cls)
            elif cls.ring==tcls.ring:
                cls = PolynomialRing[cls.variables+tcls.variables, cls.ring]
                r = r.as_algebra(cls)
                t = t.as_algebra(cls)
                r = mul_POLY_POLY(r, t, cls)
            else:
                raise NotImplementedError(`r, t`)
        if not r.data:
            return cls.Number(1)
        return r

    @classmethod
    def Pow(cls, base, exp):
        r = cls.__pow__(base, exp)
        if r is NotImplemented:
            raise NotImplementedError(`base,exp`)
        return r

    @classmethod
    def convert_coefficient(cls, obj, typeerror=True):
        if not isinstance(obj, cls.ring):
            r = cls.ring.convert_coefficient(obj, typeerror=False)
            if r is not None:
                return r
            r = cls.ring.convert(obj, typeerror=False)
            if r is not None:
                return r
        if typeerror:
            raise TypeError('%s.convert_coefficient: failed to convert %s instance'\
                            ' to coefficient algebra, expected int|long object'\
                            % (cls.__name__, obj.__class__.__name__))
        else:
            return NotImplemented

    def as_verbatim(self):
        data = self.data
        symbols = [Verbatim.convert(v) for v in self.variables]
        terms = []
        for exps in sorted(data.keys(), reverse=True):
            coeff = data[exps]
            if coeff==1:
                factors = []
            else:
                factors = [Verbatim.convert(coeff)]
            if not isinstance(exps, tuple):
                exps = exps,
            for s,e in zip(symbols, exps):
                if e:
                    if e==1:
                        factors.append(s)
                    else:
                        factors.append(s**e) #XXX: ????
            if not factors:
                terms.append(Verbatim.convert(coeff))
            elif len(factors)==1:
                terms.append(factors[0])
            else:
                terms.append(Verbatim(MUL,tuple(factors)))
        if not terms:
            return Verbatim.convert(0)
        if len(terms)==1:
            return terms[0]
        return Verbatim(ADD, tuple(terms))

    def as_algebra(self, target_cls):
        cls = self.__class__
        if cls is target_cls:
            return self
        if issubclass(target_cls, PolynomialRing):
            target_ring = target_cls.ring
            if cls.variables == target_cls.variables:
                data = dict([(e,target_ring.convert(c)) for e,c in self.data.iteritems()])
                return target_cls(data)
            if cls.ring == target_ring:
                new_cls = PolynomialRing[set(cls.variables + target_cls.variables), target_ring]
                new_variables = list(new_cls.variables)
                indices = [new_variables.index(v) for v in cls.variables]
                data = {}
                n = new_cls.nvars
                for e,c in self.data.iteritems():
                    new_e = [0] * n
                    if not isinstance(e, tuple):
                        e = e,
                    for i,j in enumerate(indices):
                        new_e[j] = e[i]
                    data[AdditiveTuple(new_e)] = c
                return new_cls(data)
        return NotImplemented

    def __neg__(self):
        head, data = self.pair
        return type(self)(head, dict([(e,-c) for e, c in data.iteritems()]))
        
    def __add__(self, other):
        cls = self.__class__
        other_cls = other.__class__
        if cls == other_cls and self.head is other.head:
            return add_POLY_POLY(self, other, cls)
        other = self.convert(other)
        if other is NotImplemented:
            return other
        other_cls = other.__class__
        if cls == other_cls and self.head is other.head:
            return add_POLY_POLY(self, other, cls)
        elif self.nvars < other.nvars:
            return other + self
        return self.head.add(cls, self, other)

    __radd__ = __add__

    def __mul__(self, other):
        cls = self.__class__
        other_cls = other.__class__
        if cls == other_cls and self.head is other.head:
            return mul_POLY_POLY(self, other, cls)
        other2 = self.convert_coefficient(other, typeerror=False)
        if other2 is not NotImplemented:
            return mul_POLY_COEFF(self, other2, cls)
        other = self.convert(other, typeerror=False)
        if other is NotImplemented:
            return other
        other_cls = other.__class__
        if cls == other_cls and self.head is other.head:
            return mul_POLY_POLY(self, other, cls)
        elif self.nvars < other.nvars:
            return other * self
        return self.head.commutative_mul(cls, self, other)

    __rmul__ = __mul__

    def __divmod__(self, other):
        cls = self.__class__
        other_cls = other.__class__
        if cls == other_cls:        
            if cls.nvars==1:
                return divmod_POLY1_POLY1_SPARSE(self, other, cls)
        return NotImplemented

    def __div__(self, other):
        cls = self.__class__
        r = self.convert_coefficient(other, typeerror=False)
        if r is not NotImplemented:
            return mul_POLY_COEFF(self, div(1, r), cls)        
        return divmod(self, other)[0]

    def __mod__(self, other):
        return divmod(self, other)[1]
    
    def __pow__(self, other):
        if isinstance(other, type(self)):
            if other.head is NUMBER:
                other = other.data
            if other.head is SPARSE_POLY:
                data = other.data
                if len(data)==1:
                    exps, coeff = data.items()[0]
                    if exps.data==[0]*len(exps):
                        other = coeff
        if isinstance(other,(int, long)):
            return pow_POLY_INT(self, other, self.__class__)
        print `self`, `other.pair`
        return NotImplemented

    def __call__(self, *args):
        # XXX: Use Horner scheme.
        r = 0
        if self.nvars==1:
            for exps, coeff in self.data.iteritems():
                r = coeff * args[0]**exps + r
        else:
            for exps, coeff in self.data.iteritems():
                r = reduce(lambda x,y: x*y, [a**e for a,e in zip(args,exps)], coeff) + r
        return r

    @property
    def degree(self):
        d = self._degree
        if d is None:
            data = self.data
            if not data:
                self._degree = d = 0
            elif self.nvars==0:
                self._degree = d = 0
            elif self.nvars==1:
                self._degree = d = max(data.keys())
            else:
                self._degree = d = map(max,zip(*data.keys()))
        return d

    @property
    def ldegree(self):
        d = self._ldegree
        if d is None:
            data = self.data
            if not data:
                self._ldegree = d = 0
            elif self.nvars==0:
                self._ldegree = d = 0
            elif self.nvars==1:
                self._ldegree = d = min(data.keys())
            else:
                self._ldegree = d = map(min,zip(*data.keys()))
        return d

    @property
    def coeff(self):
        data = self.data
        nvars = self.nvars
        if nvars==1:
            if data:
                return data[self.degree]
            return self.zero
        raise NotImplementedError(`self,nvars`)

    def diff(self, index=0): # TODO: don't use index as the order of variables is automatically sorted
        if self.nvars==0:
            return self.zero
        head, data = self.pair
        d = {}
        if self.nvars==1:
            assert index==0,`index`
            for exps, coeff in data.iteritems():
                if exps:
                    d[exps-1] = coeff * exps
        else:
            assert 0<=index<self.nvars, `index`
            for exps, coeff in data.iteritems():
                e = exps[index]
                if e:
                    d[exps.add(index,-1)] = coeff * e
        return type(self)(head, d)

    def variable_diff(self, variable, order=1):
        try:
            index = list(self.variables).index(variable)
        except ValueError:
            index = None
        if index is not None:
            return self.head.diff_index(type(self), self.data, self, index, order=order)
        raise NotImplementedError(`self.variables, variable, index`)

    def variable_integrate(self, variable, *bounds):
        """ Return integral over variable.

        poly.variable_integrate(x) -> indefinite integral
        poly.variable_integrate(x, a, b) -> definite integral
        """
        try:
            index = list(self.variables).index(variable)
        except ValueError:
            index = None
        if index is not None:
            indef_integral = self.head.integrate_indefinite_index(type(self), self.data, self, index)
            if bounds:
                low, high = bounds
                return indef_integral.variable_subs(variable, high) - indef_integral.variable_subs(variable, low)
            return indef_integral
        raise NotImplementedError(`self.variables, variable, index`)
    
    def variable_subs(self, variable, newexpr):
        """ Substitute variable with newexpr that must be instance of the same ring.
        """
        cls = type(self)
        newexpr = cls(newexpr)
        try:
            index = list(self.variables).index(variable)
        except ValueError:
            index = None
        if index is not None:
            head, data = self.pair
            result = cls.Number(0)
            variables = cls.variables
            for exps, coeff in data.iteritems():
                term = cls.Number(1)
                for i,exp in enumerate(exps):
                    if exp:
                        if i==index:
                            term *= newexpr**exp
                        else:
                            term *= cls.Symbol(variables[i])**exp
                result += term * cls.Number(coeff)
            return result
        raise NotImplementedError(`self.variables, variable, index`)

def divmod_POLY1_POLY1_SPARSE(lhs, rhs, cls):
    d2 = rhs.degree
    c2 = rhs.coeff
    if not c2:
        raise ZeroDivisionError, "polynomial division"
    other_iter = rhs.data.iteritems
    dq = {}
    dr = dict(lhs.data)
    dseq = [0] + dr.keys()
    dr_get = dr.get
    dseq_append = dseq.append
    dseq_remove = dseq.remove
    while 1:
        d1 = max(dseq)
        if d1 < d2:
            return cls(SPARSE_POLY, dq), cls(SPARSE_POLY, dr)
        c1 = dr[d1]
        e = d1 - d2
        c = div(c1, c2)
        dq[e] = c
        nc = -c
        for e0,c0 in other_iter():
            e0 = e0 + e
            c0 = nc * c0
            b = dr_get(e0)
            if b is None:
                dr[e0] = c0
                dseq_append(e0)
            else:
                b = b + c0
                if b:
                    dr[e0] = b
                else:
                    del dr[e0]
                    dseq_remove(e0)

def divmod_POLY1_POLY1_DENSE(lhs, rhs, cls):
    if not rhs.coeff:
        raise ZeroDivisionError, "polynomial division"
    n = lhs.degree
    nv = rhs.degree
    u = [0]*(n+1)
    v = [0]*(nv+1)
    for e,c in lhs.data.iteritems():
        u[e] = c
    for e,c in rhs.data.iteritems():
        v[e] = c
    r = list(u)
    q = [0] * (len(r)+1)
    for k in range(n-nv, -1, -1):
        q[k] = div(r[nv+k], v[nv])
        for j in range(nv+k-1, k-1, -1):
            r[j] -= q[k]*v[j-k]
    for j in range(nv, n+1, 1):
        r[j] = 0
    q, r = cls(q), cls(r)
    return q, r

def pow_POLY_INT(base, exp, cls):
    if exp==0:
        return cls.one
    if exp==1:
        return base
    if exp < 0:
        return NotImplemented
    data = base.data
    if len(data)==1:
        exps, coeff = data.items()[0]
        return cls(SPARSE_POLY, {exps * exp: coeff ** exp})
    nvars = cls.nvars
    d = {}
    items = data.items()
    m = len(data)
    for k,c_kn in multinomial_coefficients(m, exp).iteritems():
        new_exps = AdditiveTuple((0,)*nvars)
        new_coeff = c_kn
        for i,e in enumerate(k):
            if e:
                exps, coeff = items[i]
                new_exps += exps * e
                new_coeff *= coeff ** e
        b = d.get(new_exps)
        if b is None:
            d[new_exps] = new_coeff
        else:
            c = b + new_coeff
            if c:
                d[new_exps] = c
            else:
                del d[new_exps]
    return cls(SPARSE_POLY, d)
    

def add_POLY_POLY(lhs, rhs, cls):
    d = dict(lhs.data)
    for exps, coeff in rhs.data.iteritems():
        b = d.get(exps)
        if b is None:
            d[exps] = coeff
        else:
            c = b + coeff
            if c:
                d[exps] = c
            else:
                del d[exps]
    return cls(SPARSE_POLY, d)

def iadd_POLY_POLY(lhs, rhs, cls):
    d = lhs.data
    for exps, coeff in rhs.data.iteritems():
        b = d.get(exps)
        if b is None:
            d[exps] = coeff
        else:
            c = b + coeff
            if c:
                d[exps] = c
            else:
                del d[exps]
    

def mul_POLY_POLY(lhs, rhs, cls):
    d = {}
    for exps1, coeff1 in lhs.data.iteritems():
        for exps2, coeff2 in rhs.data.iteritems():
            exps = exps1 + exps2
            coeff = coeff1 * coeff2
            b = d.get(exps)
            if b is None:
                d[exps] = coeff
            else:
                c = b + coeff
                if c:
                    d[exps] = c
                else:
                    del d[exps]
    return cls(SPARSE_POLY, d)

def mul_POLY_COEFF(lhs, rhs, cls):
    d = {}
    for exps, coeff in lhs.data.iteritems():
        b = d.get(exps)
        if b is None:
            d[exps] = coeff * rhs
        else:
            c = b + coeff * rhs
            if c:
                d[exps] = c
            else:
                del d[exps]
    return cls(SPARSE_POLY, d)





# Author: Pearu Peterson
# Created: April 2011

from __future__ import division
import cPickle as pickle
import os
import random
import time
from collections import defaultdict

from ...matrices import Matrix
from .io import load_stoic_from_sbml, load_stoic_from_text
from .utils import objsize

class SteadyFluxAnalyzer(object):
    """ Base class for analyzing the steady state of a metabolic network.


    Attributes
    ----------
    species : list
      A list of species names or row names.
    reaction : list
      A list of reaction names or column names.
    stoichiometry : Matrix
      A stoichiometric matrix.

    """

    def __init__(self, source,
                 discard_boundary_species=False,
                 add_boundary_fluxes=False,
                 split_bidirectional_fluxes = False,
                 growth_factor = 1,
                 growth_shuffle = True,
                 ):
        """
        Parameters
        ----------
        source : str
          A file name to sbml file or a string with reaction definitions.
          For internal processing source can also be tuple.

        discard_boundary_species : bool

          When True then discard species that are reactants or
          products of the full network. The corresponding
          stoichiometry system will be open. For example, in a
          reaction ``A -> B -> C`` the species A and C are reactant
          and product of the system and after discarding A and C, the
          system will be open: ``-> B ->`` .  In the case of a
          reaction `` -> A -> B + C`` the system will be made open by
          adding new reactions `` B-> `` and `` C -> ``.

        add_boundary_fluxes : bool

          When True then add boundary fluxes to boundary
          species.  The corresponding stoichiometry system will be
          open.  For example, in a reaction ``A -> B -> C`` the
          species A and C are reactant and product of the system and
          after introducing boundary fluxes, the system will be open:
          ``-> A -> B -> C ->``.  New flux names start with prefix
          ``BR_``.

        split_bidirectional_fluxes : bool
          When True the bidirectional fluxes are split into two unidirectional fluxes.
          For example, the system ``A<=>B`` is treated as ``A=>B and B=>A``.

        growth_factor : int
          Increase the size of network by growth_factor.
        growth_shiffle : bool
          When True then shuffle the rows and columns of increased network.

        See also
        --------
        load_stoic_from_sbml, load_stoic_from_text, discard_boundary_species, add_boundary_fluxes
        """
        if isinstance (source, str):
            if os.path.isfile (source) or (source.count ('\n')==0 and '=' not in source):
                stoic_dict, species, reactions, species_info, reactions_info = \
                    load_stoic_from_sbml(source, split_bidirectional_fluxes=split_bidirectional_fluxes)
            else:
                stoic_dict, species, reactions, species_info, reactions_info = \
                    load_stoic_from_text(source, split_bidirectional_fluxes=split_bidirectional_fluxes)
        elif isinstance (source, tuple):
            stoic_dict, species, reactions, species_info, reactions_info = source
        else:
            raise TypeError ('expected source to be str or tuple but got %r' % (type(source)))
        if discard_boundary_species:
            stoic_dict, species, reactions, species_info, reactions_info = \
                self.discard_boundary_species(stoic_dict, species, reactions, species_info, reactions_info)
        if add_boundary_fluxes:
            stoic_dict, species, reactions, species_info, reactions_info = \
                self.add_boundary_fluxes(stoic_dict, species, reactions, species_info, reactions_info)
        if growth_factor>1:
            stoic_dict, species, reactions, species_info, reactions_info = \
                self.apply_growth(stoic_dict, species, reactions, species_info, reactions_info, growth_factor, growth_shuffle)


        self.source = source
        self.options = (discard_boundary_species, add_boundary_fluxes, split_bidirectional_fluxes, growth_factor, growth_shuffle)
        self.source_data = stoic_dict, species, reactions, dict(species_info), dict(reactions_info)

        self._stoichiometry = None
        self.compute_kernel_GJE_data = None
        self.compute_kernel_SVD_data = None

    @property
    def species(self): return self.source_data[1]
    @property
    def reactions(self): return self.source_data[2]
    @property
    def reactions_info(self): return self.source_data[4]
    @property
    def stoichiometry(self):
        if self._stoichiometry is None:
            self._stoichiometry = Matrix(len (self.species), len (self.reactions), self.source_data[0])
        return self._stoichiometry

    def _get_pickle_file_name(self, file_name):
        for ext in ['', '.pkl', '.pickle']:
            if os.path.isfile(file_name+ext):
                file_name = file_name + ext
                break
        if os.path.splitext(file_name)[1]=='':
            file_name = file_name + '.pkl'
        return file_name

    def save(self, file_name):
        """ Save instance data to file_name.
        """
        file_name = self._get_pickle_file_name(file_name)
        dirname = os.path.dirname (file_name)
        if os.path.isfile(file_name):

            f = open(file_name, 'rb')
            try:
                data = pickle.load(f)
            except Exception, msg:
                print 'Failed to load %r: %s' % (file_name, msg)
                data = []
            f.close()
        else:
            data = []
        if not isinstance(data, list):
            data = []

        key = (self.source, self.options)
        value = {}
        for a in dir(self):
            if a.endswith('_data') or a.endswith('_elapsed'):
                value[a] = getattr(self, a)
        
        data_file_name = None
        for key0, data_file_name0 in data:
            if key0==key:
                data_file_name = os.path.join(dirname, os.path.basename(data_file_name0))
                break
        if data_file_name is None:
            base, ext = os.path.splitext(file_name)
            data_file_name = '%s_%s%s' % (os.path.basename(base), len(data), ext)
            data.append((key, data_file_name))

            f = open(file_name, 'wb')
            pickle.dump(data, f)
            f.close()

        f = open(data_file_name, 'wb')
        pickle.dump(value, f)
        f.close()
        print 'Succesfully wrote data to', data_file_name
        
    def load(self, file_name):
        """ Load instance data from file_name.
        """
        file_name = self._get_pickle_file_name(file_name)
        dirname = os.path.dirname (file_name)
        if os.path.isfile(file_name):
            f = open(file_name, 'rb')
            try:
                data = pickle.load(f)
            except Exception, msg:
                print 'Failed to load %r: %s' % (file_name, msg)
                data = []
            f.close()
        else:
            return
        data_file_name = None
        key = (self.source, self.options)
        for key0, data_file_name0 in data:
            if len(key)==len(key0)+1:
                # for backward compatibility
                key0 = key0[0], key0[1][:2] + (False,) + key0[1][2:]
                
            if key0==key:
                data_file_name = os.path.join(dirname, os.path.basename(data_file_name0))
                break
        if data_file_name is None:
            return
        f = open(data_file_name, 'rb')
        value = pickle.load(f)
        f.close()
        print 'Succesfully loaded data from', data_file_name
        self._stoichiometry = None
        for a,v in value.iteritems():
            setattr(self, a, v)
        return True

    def __repr__(self):
        return '%s((%r, %r, %r, %r, %r))' % (self.__class__.__name__,
                                             self.stoichiometry, self.species, self.reactions,
                                             self.species_info, self.reactions_info)

    def __str__(self, max_nrows=20, max_ncols=20):
        mat = self.label_matrix(self.stoichiometry, self.species, self.reactions)
        return mat.__str__(max_nrows=max_nrows, max_ncols=max_ncols)

    def label_matrix(self, matrix, row_labels, col_labels):
        """ Return a matrix with labeled rows and columns.
        """
        m, n = matrix.shape
        new_matrix = Matrix(m+1, n+1)
        new_matrix[1:, 1:] =  matrix
        for i, r in enumerate (col_labels):
            new_matrix[0,i+1] = r
        for i, r in enumerate (row_labels):
            new_matrix[i+1,0] = r
        new_matrix[0,0] = ' '
        return new_matrix

    @property
    def shape(self):
        return self.stoichiometry.shape

    @property
    def sparsity(self):
        m,n = self.stoichiometry.shape
        return 1-len (self.stoichiometry.data)/(m*n)

    @property
    def rank (self):
        self.compute_kernel_GJE(use_cache=True)        
        return len(self.dependent_variables)

    def get_splitted_reactions (self):
        """ Split reactions to a list of dependent and independent
        reactions.

        Returns
        -------
        leading_cols_canditates, trailing_cols_canditates : list
          Lists of integers.
        """
        reactions = self.reactions
        independent_cols = []
        for reaction_id in reactions:
            if reaction_id.startswith('BR_'):
                independent_cols.append(reactions.index(reaction_id))
        leading_cols_canditates = sorted(set(range(len(reactions))).difference(independent_cols))
        trailing_cols_canditates = independent_cols
        return leading_cols_canditates, trailing_cols_canditates

    @classmethod
    def apply_growth(cls, stoic_dict, species, reactions, species_info, reactions_info, growth_factor=1, growth_shuffle=True):
        """ Increase the size of network by a growth_factor.

        Useful for testing.
        """
        if growth_factor==1:
            return stoic_dict, species, reactions, species_info, reactions_info
        m0, n0 = len(species), len(reactions)
        m1, n1 = m0*growth_factor, n0*growth_factor
        species1 = []
        reactions1 = []
        for i in range (growth_factor):
            for s in species:
                species1.append('%s_%s' % (s, i))
            for s in reactions:
                reactions1.append('%s_%s' % (s, i))
        rindices = range(m1)
        indices = range(n1)
        if growth_shuffle:
            random.shuffle (rindices)
            random.shuffle (indices)
            species1 = [species1[i] for i in rindices]
            reactions1 = [reactions1[i] for i in indices]

        stoic_dict1 = {}
        for k in range (growth_factor):
            for (i,j), v in stoic_dict.iteritems():
                stoic_dict1[rindices[i+ k*m0], indices[j+ k*n0]] = v 
        return stoic_dict1, species1, reactions1, {}, {}

    def compute_kernel_GJE(self,
                           leading_row_selection='sparsest first',
                           leading_column_selection='sparsest first',
                           dependent_candidates = None,
                           force = False,
                           use_cache = None
                           ):
        """ Compute the kernel of stoichiometric matrix via GJE routine.

        Parameters
        ----------
        leading_row_selection, leading_column_selection : str
          Specify parameters to get_gauss_jordan_elimination_operations method.

        dependent_candidates : list

          Specify the list of dependent fluxes. Note that this is
          considered only as a suggestion. The actual list of
          dependent fluxes may be different.

        See also
        --------
        MatrixBase.get_gauss_jordan_elimination_operations
        """
        if use_cache and self.compute_kernel_GJE_data is not None:
            options = self.compute_kernel_GJE_parameters_data
        else:
            options = leading_row_selection, leading_column_selection, dependent_candidates
        if self.compute_kernel_GJE_data is not None and self.compute_kernel_GJE_parameters_data != options:
            force = True
        if self.compute_kernel_GJE_data is None or force:
            if dependent_candidates is None:
                leading_cols_candidates, trailing_cols_candidates = self.get_splitted_reactions()
            else:
                leading_cols_candidates = [self.reactions.index (r) for r in dependent_candidates]
                trailing_cols_candidates = [self.reactions.index (r) for r in self.reactions if r not in dependent_candidates]
            start = time.time ()
            result = self.stoichiometry.get_gauss_jordan_elimination_operations(leading_cols=leading_cols_candidates,
                                                                                trailing_cols = trailing_cols_candidates,
                                                                                leading_row_selection=leading_row_selection,
                                                                                leading_column_selection=leading_column_selection)
            end = time.time()
            self.compute_kernel_GJE_data = result
            self.compute_kernel_GJE_elapsed = end - start

            independent_variables = []
            dependent_variables = []
            for k, variable in enumerate(self.reactions):
                if k in result[3]:
                    dependent_variables.append(variable)
                else:
                    independent_variables.append(variable)
            self.variables_data = dependent_variables, independent_variables
            self.compute_kernel_GJE_parameters_data = options

            
    @property
    def dependent_variables(self):
        return self.variables_data[0]

    @property
    def independent_variables(self):
        return self.variables_data[1]


    def get_sorted_reactions(self):
        from sympycore.matrices.linalg import  get_rc_map
        self.compute_kernel_GJE(use_cache=True)
        start = time.time ()
        gj, row_operations, leading_rows, leading_cols, zero_rows = self.compute_kernel_GJE_data
        l = []
        rows = get_rc_map(gj.data)
        for i0,j0 in zip (leading_rows, leading_cols):
            row = rows[i0]
            row.remove (j0)
            if row:
                l.append ((len(row),-min(row), self.reactions[j0]))
            else:
                l.append ((len(row),0, self.reactions[j0]))
        reactions = [s for i,i1,s in sorted(l, reverse=True)]
        reactions += [s for s in self.reactions if s not in reactions]
        end = time.time ()
        self.get_sorted_reactions_elapsed = end-start
        return reactions

    def get_kernel_GJE(self, reactions=None):
        """ Return the kernel K from GJE routine.
        
        Notes
        -----
        The steady state solution is given by ``reactions = K * indep_variables``
        where ``indep_variables = reactions[-K.shape[1]:]``

        Parameters
        ----------
        reactions : list
          When specified then the reaction list defines the order of flux rows.
          Otherwise the rows are order by the sparsity.

        Returns
        -------
        fluxes, indep_fluxes, kernel : list, list, Matrix
        """
        from sympycore.matrices.linalg import  get_rc_map
        self.compute_kernel_GJE(use_cache=True)
        gj, row_operations, leading_rows, leading_cols, zero_rows = self.compute_kernel_GJE_data
        if reactions is None:
            reactions = self.get_sorted_reactions()
        start = time.time ()
        kernel = Matrix(len(reactions), len(reactions)-len(leading_cols))
        indep_vars = [r for r in reactions if r in self.independent_variables]
        n = len (indep_vars)
        rows = get_rc_map(gj.data)
        def indep_index (r):
            try: return indep_vars.index (r)
            except ValueError: pass
        indep_indices = []
        indices = []
        for r in self.reactions:
            indices.append(reactions.index(r))
            indep_indices.append(indep_index (r))

        for i0,j0 in zip(leading_rows, leading_cols):
            i = indices[j0]
            for j1 in rows[i0]:
                if j1!=j0:
                    j = indep_indices[j1]
                    kernel[i,j] = -gj.data[(i0,j1)]

        for j0 in range (len (self.reactions)):
            if j0 not in leading_cols:
                i = indices[j0]
                j = indep_indices[j0]
                kernel[i,j] = 1
        end = time.time()
        self.get_kernel_GJE_elapsed = end-start
        return reactions, indep_vars, kernel

    def get_relation_GJE(self, reactions=None):
        """ Return relation matrix R from GJE routine.

        Notes
        -----
        The steady state solution is given by ``dep_fluxes = R * indep_fluxes``

        Parameters
        ----------
        reactions : list
          When specified then the reaction list defines the order of rows.
          Otherwise the rows are order by the sparsity.

        Returns
        -------
        dep_fluxes, indep_fluxes, rmatrix : list, list, Matrix

        See also
        --------
        get_relation_SVD
        """
        reactions, indep_reactions, kernel = self.get_kernel_GJE(reactions=reactions)
        start = time.time()
        rank = kernel.shape[0] - kernel.shape[1]
        dep_reactions = [r for r in reactions if r not in indep_reactions]
        r = kernel[:rank]
        end = time.time()
        self.get_relation_GJE_elapsed = end-start
        return dep_reactions, indep_reactions, r

    def get_dense_stoichiometry(self, species, reactions):
        import numpy
        r = numpy.zeros (self.stoichiometry.shape, dtype=float)
        data = self.stoichiometry[:].data # todo: optimize when not transposed
        for (i,j), v in data.iteritems ():
            i1 = species.index (self.species[i])
            j1 = reactions.index (self.reactions[j])
            r[i1,j1] = v
        return r

    def compute_kernel_SVD(self):
        """Compute the kernel of stoichiometric matrix via SVD routine.
        """
        import numpy
        if self.compute_kernel_SVD_data is None:
            array = self.get_dense_stoichiometry (self.species, self.reactions)
            start = time.time ()
            U, s, V = numpy.linalg.svd(array)
            end = time.time()
            self.compute_kernel_SVD_data = U, s, V
            self.compute_kernel_SVD_elapsed = end - start

    def get_kernel_SVD(self, reactions=None):
        """Return the kernel K from SVD routine.

        Notes
        -----
        The steady state solution is given by ``reactions = K * parameters``.
        
        Parameters
        ----------
        reactions : list
          When specified then the reaction list defines the order of rows.
          Otherwise the rows are order by the sparsity.

        Returns
        -------
        reactions, kernel : list, Matrix
        """
        import numpy
        self.compute_kernel_SVD()
        if reactions is None:
            reactions = self.get_sorted_reactions()
        start = time.time ()
        U, s, V = self.compute_kernel_SVD_data
        Vker = V[self.rank:].T
        Vker1 = numpy.zeros (Vker.shape, dtype=float)
        for i, r in enumerate(reactions):
            Vker1[i] = Vker[self.reactions.index(r)]
        end = time.time()
        self.get_kernel_SVD_elapsed = end-start
        return reactions, Vker1

    def get_relation_SVD(self, reactions=None):
        """ Return relation matrix R from SVD routine.

        Notes
        -----
        The steady state solution is given by ``dep_fluxes = R * indep_fluxes``

        Parameters
        ----------
        reactions : list
          When specified then the reaction list defines the order of rows.
          Otherwise the rows are order by the sparsity.

        Returns
        -------
        dep_fluxes, indep_fluxes, rmatrix : list, list, numpy.ndarray
        
        See also
        --------
        get_relation_GJE
        """
        import numpy
        reactions, kernel = self.get_kernel_SVD(reactions=reactions)
        start = time.time ()
        rank = kernel.shape[0]-kernel.shape[1]
        dep_kernel = kernel[:rank]
        indep_kernel = kernel[rank:]
        dep_reactions = reactions[:rank]
        indep_reactions = reactions[rank:]
        r = numpy.dot(dep_kernel, numpy.linalg.inv(indep_kernel))
        end = time.time ()
        self.get_relation_SVD_elapsed = end-start
        return dep_reactions, indep_reactions, r

    def get_relation_SVD_error(self, reactions=None):
        """ Compute maximal relative flux error of the SVD relation routine.

        Notes
        -----
        See the paper for definition.

        Returns
        -------
        rerr : float
        """
        import numpy
        dep_reactions_GJE, indep_reactions_GJE, relation_GJE = self.get_relation_GJE (reactions=reactions)
        dep_reactions_SVD, indep_reactions_SVD, relation_SVD = self.get_relation_SVD (reactions=reactions)
        assert dep_reactions_GJE==dep_reactions_GJE
        assert indep_reactions_GJE==indep_reactions_GJE
        dense_relation_GJE = numpy.zeros(relation_GJE.shape, dtype=float)
        for (i,j),v in relation_GJE.data.iteritems ():
            dense_relation_GJE[i,j]=v
        abserrdata = abs(relation_SVD - dense_relation_GJE).sum (axis=1)
        absdata = numpy.maximum(numpy.maximum(abs(relation_SVD),
                                              abs(dense_relation_GJE)),
                                numpy.ones(relation_SVD.shape)).sum (axis=1)
    
        return max(abserrdata/absdata)

    @classmethod
    def add_boundary_fluxes(cls, matrix, species, reactions, species_info, reactions_info):
        """ Make network open by adding new columns that correspond to
        the boundary fluxes.
        
        Boundary fluxes are defines as fluxes from boundary species
        that enter or leave the network. The names of boundary fluxes
        start with prefix ``BR_``.
        """
        for specie_index in range(len(species)):
            reaction_indices = [j for i,j in matrix if i==specie_index]
            if len (reaction_indices)>1 and\
                not (len(reaction_indices)==2 and len(set(reaction_indices))==1): # to catch polymerization reactions
                continue
            reaction_index = reaction_indices[0]
            stoichiometry = matrix[specie_index, reaction_index]
            specie_id = species[specie_index]
            new_reaction_id = 'BR_%s' % (specie_id)
            new_reaction_index = len(reactions)
            reactions.append(new_reaction_id)
            new_stoichiometry = -1 if stoichiometry>0 else 1
            matrix[specie_index, new_reaction_index] = new_stoichiometry
            if new_stoichiometry>0:
                reactions_info[new_reaction_id]['products'].append(specie_id)
            else:
                reactions_info[new_reaction_id]['reactants'].append(specie_id)
        return matrix, species, reactions, species_info, reactions_info

    @classmethod
    def discard_boundary_species(cls, matrix, species, reactions, species_info, reactions_info):
        """ Make the network open by discarding rows that correspond
        to boundary species.

        Boundary species are defined as the products or reactants of the network.
        """
        boundary_species = []
        extra_reactions = []
        for specie_index in range(len(species)):
            reaction_indices = [j for i,j in matrix if i==specie_index]
            if len(reaction_indices)>1:
                continue
            reaction_index = reaction_indices[0]
            reaction_id = reactions[reaction_index]
            stoichiometry = matrix[specie_index, reaction_index]
            boundary_species.append(specie_index)

        i = 0
        new_matrix = {}
        new_species = []
        for specie_index in range(len(species)):
            if specie_index in boundary_species:
                continue
            new_species.append(species[specie_index])
            for i0, reaction_index in matrix:
                if i0 != specie_index:
                    continue
                new_matrix[i, reaction_index] = matrix[specie_index, reaction_index]
            i += 1
        species = new_species
        matrix = new_matrix

        return matrix, species, reactions, species_info, reactions_info

    @property
    def sparsity_kernel_GJE(self):
        reactions, indep_reactions, kernel = self.get_kernel_GJE()
        return 1-len (kernel.data)/(kernel.shape[0]*kernel.shape[1])

    @property
    def sparsity_kernel_SVD(self):
        reactions, kernel = self.get_kernel_SVD()
        return 1-(abs(kernel)>1e-3).sum()/kernel.size

    @property
    def condition_number (self):
        self.compute_kernel_SVD()
        U, s, V = self.compute_kernel_SVD_data        
        return s[0]/s[self.rank-1]

    def matrix_plot(self, matrix, figure_name='matrix_plot.pdf'):
        import numpy
        from matplotlib import pylab
        def _blob(x,y,area,colour):
            hs = numpy.sqrt(area) / 2
            xcorners = numpy.array([x - hs, x + hs, x + hs, x - hs])
            ycorners = numpy.array([y - hs, y - hs, y + hs, y + hs])
            pylab.fill(xcorners, ycorners, colour, edgecolor=colour)
        reenable = False
        if pylab.isinteractive():
            pylab.ioff()
        pylab.clf()
        
        maxWeight = 2**numpy.ceil(numpy.log(numpy.max(numpy.abs(matrix)))/numpy.log(2))
        height, width = matrix.shape
        pylab.fill(numpy.array([0,width,width,0]),numpy.array([0,0,height,height]),'white')
        pylab.axis('off')
        pylab.axis('equal')
        for x in xrange(width):
            for y in xrange(height):
                _x = x+1
                _y = y+1
                w = matrix[y,x]
                if w > 0:
                    _blob(_x - 0.5, height - _y + 0.5, 0.2,'#0099CC')
                elif w < 0:
                    _blob(_x - 0.5, height - _y + 0.5, 0.2,'#660000')

        if reenable:
            pylab.ion()
        pylab.savefig(figure_name) 

    def show_timings(self):
        for a in dir (self):
            if a.endswith ('_elapsed'):
                print '%s took %s seconds' % (a[:-8], getattr (self, a))

    def show_statistics(self, *methods):
        shown = []
        print 'system size: ', self.shape
        print 'rank:', self.rank
        for method in methods:
            assert method in ['GJE', 'SVD'],`method`
            for mthprefix in ['compute_kernel_', 'get_kernel_', 'get_relation_']:
                mthname = mthprefix + method
                mth = getattr(self, mthname)
                mth()
                elapsed = getattr (self, mthname+'_elapsed')
                print '  %s took %.3f seconds' % (mthname, elapsed)
                shown.append(mthname)

        for a in dir (self):
            if a.endswith ('_elapsed'):
                mthname = a[:-8]
                if mthname not in shown:
                    print '%s took %s seconds' % (mthname, getattr (self, a))
            elif a.endswith ('_data'):
                mthname = a[:-5]
                print '%s consumed %s bytes of memory' % (mthname, objsize(getattr (self, a)))

        if self.compute_kernel_GJE_data is not None:
            print 'compute_kernel_GJE performed %s row operations' % (len(self.compute_kernel_GJE_data[1]))




from sympycore import *
from sympycore.calculus.relational import *
x = Symbol('x')

A = Assumptions

def test_inequalities():

    assert A().check(2 < 3) is True
    assert A().check(Rational(2,3) < Rational(7,2)) is True
    assert A().check(pi**2 > 0) is True
    assert A().check(pi**2 < 0) is False
    assert A().check(3+4*pi > 0) is True
    assert A().check(3-pi > 0) is None
    assert A().check(-4*pi < 0) is True
    assert A().check(-4*pi > 0) is False
    assert A().check(-4*pi**Rational(1,2) < 0) is True
    assert A().check(-4*pi**Rational(1,2) > 0) is False
    assert A().check(-4*pi**2 - 1 < 0) is True
    assert A([x > 3]).check(x > 2) is True
    assert A([x > 3]).check(x > 3) is True
    assert A([x > 3]).check(x > 4) is None
    #assert A([x > 3]).check(x < 3) is False
    assert A([x > 3]).check(x > 2) is True

def XXXtest_double_inequalities():
    assert A([x>0, x<1]).check(x < 1) is True
    assert A([x>0, x<1]).check(x > 0) is True
    assert A([x>0, x<1]).check(x > 2) is False
    assert A([x>0, x<1]).check(x < -1) is False




from sympycore import *

A = Calculus
x,y,z = map(A.Symbol,'xyz')
f = A.convert(1.2)

def test_ops():
    assert f+1==A.convert(2.2)
    assert f+1.2==A.convert(2.4)

    assert `f+x` in ["Calculus('1.2 + x')", "Calculus('x + 1.2')"],`f+x`
    assert `1.2+x` in ["Calculus('1.2 + x')", "Calculus('x + 1.2')"],`1.2+x`
    #assert `f+pi`=="Calculus('4.34159265359')"
    #assert `1.2+pi`=="Calculus('4.34159265359')"




import os
import tempfile
import pickle

from sympycore import *

def pickler(obj):
    fn = tempfile.mktemp()
    
    f = open(fn, 'wb')
    pickle.dump(obj, f)
    f.close()

    f = open(fn, 'rb')
    obj2 = pickle.load(f)
    f.close()
    os.remove(fn)

    return obj2

def test_pickle():

    obj = FunctionRing
    assert obj==pickler(obj)

    obj = FunctionRing('func')
    assert obj==pickler(obj)



#!/usr/bin/env python
#
# Created by Pearu Peterson in Febuary 2008
#

import os

from macros import preprocess
cwd = os.path.abspath(os.path.dirname(__file__))
targetfile_py = os.path.join(cwd,'..','sympycore','basealgebra','pairs_iops.py')

template = '''\
"""
Generated low-level arithmetic inplace functions for CollectingField.

This file is generated by the src/mk_pairs_iops.py script.
See http://sympycore.googlecode.com/ for more information.

DO NOT CHANGE THIS FILE DIRECTLY!!!
"""

from ..core import Expr
from ..arithmetic.numbers import mpqc, mpf, mpq, mpc, try_power, inttypes_set, realtypes_set, complextypes_set, numbertypes_set
from ..utils import NUMBER, TERMS, FACTORS
from .pairs_ops import div
'''

def main():
    f = open(targetfile_py, 'w')
    print >> f, template
    print >> f, preprocess('''

def return_terms(cls, pairs):
    if not pairs:
        return cls.zero
    if len(pairs)==1:
        t, c = pairs.items()[0]
        if c==1:
            return t
        if t==cls.one:
            return cls(NUMBER, c)
    @RETURN_NEW(HEAD=TERMS; DATA=pairs)

def return_factors(cls, pairs):
    if not pairs:
        return cls.one
    elif len(pairs)==1:
        t, c = pairs.items()[0]
        if c==1:
            return t
        if t==cls.one:
            return t
    @RETURN_NEW(HEAD=FACTORS; DATA=pairs)

def inplace_add(cls, obj, pairs, pairs_get, one):
    tobj = type(obj)
    if tobj is cls:
        head, data = obj.pair
        if head is NUMBER:
            if data:
                @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=one; VALUE=data; SIGN=+; USIGN=)
        elif head is TERMS:
            for t,c in data.iteritems():
                @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=t; VALUE=c; SIGN=+; USIGN=)
        else:
            @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=obj; VALUE=1; SIGN=+; USIGN=)
    @ELIF_CHECK_NUMBER(T=tobj)
        if obj:
            @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=one; VALUE=obj; SIGN=+; USIGN=)
    else:
        inplace_add(cls, cls.convert(obj), pairs, pairs_get, one)

def inplace_add2(cls, obj, coeff, pairs, pairs_get, one):
    if not coeff:
        return
    tobj = type(obj)
    if tobj is cls:
        head, data = obj.pair
        if head is NUMBER:
            value = coeff * data
            if value:
                @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=one; VALUE=value; SIGN=+; USIGN=)
        elif head is TERMS:
            for t,c in data.iteritems():
                @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=t; VALUE=coeff*c; SIGN=+; USIGN=)
        else:
            @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=obj; VALUE=coeff; SIGN=+; USIGN=)
    @ELIF_CHECK_NUMBER(T=tobj)
        value = coeff * obj
        if value:
            @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=one; VALUE=value; SIGN=+; USIGN=)
    else:
        inplace_add2(cls, cls.convert(obj), coeff, pairs, pairs_get, one)

def inplace_sub(cls, obj, pairs, pairs_get, one):
    tobj = type(obj)
    if tobj is cls:
        head, data = obj.pair
        if head is NUMBER:
            value = data
            if value:
                @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=one; VALUE=value; SIGN=-; USIGN=-)
        elif head is TERMS:
            for t,c in data.iteritems():
                @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=t; VALUE=c; SIGN=-; USIGN=-)
        else:
            @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=obj; VALUE=1; SIGN=-; USIGN=-)
    @ELIF_CHECK_NUMBER(T=tobj)
        if obj:
            @ADD_TERM_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; TERM=one; VALUE=obj; SIGN=-; USIGN=-)
    else:
        inplace_add(cls, cls.convert(obj), pairs, pairs_get, one)

def inplace_mul(cls, obj, pairs, pairs_get, try_power=try_power, NUMBER=NUMBER):
    tobj = type(obj)
    if tobj is cls:
        head, data = obj.pair
        if head is NUMBER:
            return data
        elif head is TERMS:
            if len(data)==1:
                t, number = data.items()[0]
                @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=t; VALUE=1; SIGN=+; USIGN=; NUMBER=number)
                return number
            number = 1
            @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=obj; VALUE=1; SIGN=+; USIGN=; NUMBER=number)
            return number
        elif head is FACTORS:
            number = 1
            for t, c in data.iteritems():
                @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=t; VALUE=c; SIGN=+; USIGN=; NUMBER=number)
            return number
        else:
            number = 1
            @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=obj; VALUE=1; SIGN=+; USIGN=; NUMBER=number)
            return number
    @ELIF_CHECK_NUMBER(T=tobj)
        return obj
    else:
        return inplace_mul(cls, cls.convert(obj), pairs, pairs_get)

def inplace_mul2(cls, obj, exp, pairs, pairs_get, try_power=try_power, NUMBER=NUMBER):
    if not exp:
        return cls.one
    tobj = type(obj)
    texp = type(exp)
    if tobj is cls:
        head, data = obj.pair
        if head is NUMBER:
            if texp in inttypes_set and exp < 0:
                return div(1, data ** -exp, cls)
            return data ** exp
        elif head is TERMS:
            if len(data)==1:
                t, number = data.items()[0]
                if  texp in inttypes_set and exp < 0:
                    number = div(1, number ** -exp, cls)
                else:
                    number = number ** exp
                @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=t; VALUE=exp; SIGN=+; USIGN=; NUMBER=number)
                return number
            number = 1
            @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=obj; VALUE=exp; SIGN=+; USIGN=; NUMBER=number)
            return number
        elif head is FACTORS:
            number = 1
            for t, c in data.iteritems():
                @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=t; VALUE=c*exp; SIGN=+; USIGN=; NUMBER=number)
            return number
        else:
            number = 1
            @MUL_FACTOR_VALUE_DICT(DICT=pairs; DICT_GET=pairs_get; FACTOR=obj; VALUE=exp; SIGN=+; USIGN=; NUMBER=number)
            return number
    @ELIF_CHECK_NUMBER(T=tobj)
        if  texp in inttypes_set and exp < 0:
            return div(1, obj ** -exp, cls)
        return obj ** exp
    else:
        return inplace_mul2(cls, cls.convert(obj), exp, pairs, pairs_get)

    ''', globals())

if __name__=='__main__':
    main()



import os
import tempfile
import pickle

from sympycore.arithmetic import mpq, mpqc, Infinity, mpf, mpc

def pickler(obj):
    fn = tempfile.mktemp()
    
    f = open(fn, 'wb')
    pickle.dump(obj, f)
    f.close()

    f = open(fn, 'rb')
    obj2 = pickle.load(f)
    f.close()
    os.remove(fn)

    return obj2

def test_pickle():

    obj = mpq((1,2))
    assert obj==pickler(obj)

    obj = mpqc(1,2)
    assert obj==pickler(obj)

    obj = Infinity(-1)
    assert obj==pickler(obj)

    obj = mpf('0.5')
    assert obj==pickler(obj)

    obj = mpc('0.5', '0.2')
    assert obj==pickler(obj)




__all__ = ['CALLABLE']

import re
import types
from .base import AtomicHead, heads_precedence, Expr

from ..core import init_module, classes

init_module.import_heads()
init_module.import_numbers()
init_module.import_lowlevel_operations()


_is_atomic = re.compile(r'\A\w+\Z').match

class CallableHead(AtomicHead):
    """
    CallableHead is a head for interpreting callable objects,
    data can be any callable Python object.
    """

    def is_data_ok(self, cls, data):
        if not callable(data):
            if isinstance(data, type):
                return
            return 'data instance must be callable but got %s' % (`data.pair`)
        if isinstance(data, Expr):
            if data.head==CALLABLE:
                return 'data should not have CALLABLE head'
        if not issubclass(cls, classes.FunctionRing) and cls is not classes.Verbatim:
            return 'Algebra class should be subclass of FunctionRing but got %s' % (cls.__name__)
        
    def __repr__(self): return 'CALLABLE'

    def data_to_str_and_precedence(self, cls, func):
        if isinstance(func, Expr):
            h, d = func.pair
            return h.data_to_str_and_precedence(cls, d)
        if hasattr(func, '__name__'):
            s = func.__name__
            if s=='<lambda>':
                varnames = func.func_code.co_varnames
                Algebra = cls.get_value_algebra()
                args = map(Algebra, varnames)
                expr = func(*args)
                return 'lambda %s: %s' % (', '.join(varnames), expr), heads_precedence.LAMBDA
        else:
            s = str(func)
        if _is_atomic(s):
            return s, heads_precedence.CALLABLE
        return s, 0.0 # force parenthesis

    def commutative_mul_number(self, cls, lhs, rhs):
        return term_coeff_new(cls, (lhs, rhs))

    def pow_number(self, cls, base, exp):
        return pow_new(cls, (base, exp))

    def pow(self, cls, base, exp):
        return pow_new(cls, (base, exp))

    def apply(self, cls, data, func, args):
        return data(*args)

    def fdiff(self, cls, data, expr, argument_index, order):
        if hasattr(data, 'fdiff'):
            return data.fdiff(cls, argument_index, order)
        vcls = cls.get_value_algebra()
        dcls = cls.get_differential_algebra()
        d = dcls(FDIFF, vcls(NUMBER, argument_index))**order
        return cls(APPLY, (d, (expr,)))

CALLABLE = CallableHead()



"""Provides algorithms from number theory.
"""

from ..core import init_module

init_module.import_lowlevel_operations()

from .numbers import mpq, div
from . import mpmath

__all__ = ['gcd', 'lcm', 'factorial',
           'integer_digits', 'real_digits',
           'multinomial_coefficients', 'f2q']

__docformat__ = "restructuredtext en"

def factorial(n, memo=[1, 1]):
    """Return n factorial (for integers n >= 0 only)."""
    if n < 0:
        raise ValueError("expected non-negative integer, got %r" % (n)) #pragma NO COVER
    k = len(memo)
    if n < k:
        return memo[n]
    p = memo[-1]
    while k <= n:
        p *= k
        k += 1
        if k < 100:
            memo.append(p)
    return p

def gcd(*args):
    """Calculate the greatest common divisor (GCD) of the arguments."""
    L = len(args)
    if L == 0: return 0
    if L == 1: return args[0]
    if L == 2:
        a, b = args
        while b:
            a, b = b, a % b
        return a
    return gcd(gcd(args[0], args[1]), *args[2:])

def lcm(*args):
    """Calculate the least common multiple (LCM) of the arguments."""
    L = len(args)
    if L == 0: return 0
    if L == 1: return args[0]
    if L == 2: return div(args[0], gcd(*args))*args[1]
    return lcm(lcm(args[0], args[1]), *args[2:])

# TODO: this could use the faster implementation in mpmath
def integer_digits(n, base=10):
    """Return a list of the digits of abs(n) in the given base."""
    assert base > 1
    assert isinstance(n, (int, long))
    n = abs(n)
    if not n:
        return [0]
    L = []
    while n:
        n, digit = divmod(n, base)
        L.append(int(digit))
    return L[::-1]

# TODO: this could (also?) be implemented as an endless generator
def real_digits(x, base=10, truncation=10):
    """Return ``(L, d)`` where L is a list of digits of ``abs(x)`` in
    the given base and ``d`` is the (signed) distance from the leading
    digit to the radix point.

    For example, 1234.56 becomes ``([1, 2, 3, 4, 5, 6], 4)`` and 0.001
    becomes ``([1], -2)``. If, during the generation of fractional
    digits, the length reaches `truncation` digits, the iteration is
    stopped."""
    assert base > 1
    assert isinstance(x, (int, long, mpq))
    if x == 0:
        return ([0], 1)
    x = abs(x)
    exponent = 0
    while x < 1:
        x *= base
        exponent -= 1
    integer, fraction = divmod(x, 1)
    L = integer_digits(integer, base)
    exponent += len(L)
    if fraction:
        p, q = fraction
        for i in xrange(truncation - len(L)):
            p = (p % q) * base
            if not p:
                break
            L.append(int(p//q))
    return L, exponent

def binomial_coefficients(n):
    """Return a dictionary containing pairs {(k1,k2) : C_kn} where
    C_kn are binomial coefficients and n=k1+k2.

    INPUT:
        n -- an integer

    OUTPUT:
        dict
 
    EXAMPLES:
        >>> sorted(binomial_coefficients(3).items())
        [((0, 3), 1), ((1, 2), 3), ((2, 1), 3), ((3, 0), 1)]
 
    Notice the coefficients above are the same as below:
        x**3 + 3*x**2*y + 3*x*y**2 + y^3

    """
    d = {(0, n):1, (n, 0):1}
    a = 1
    for k in xrange(1, n//2+1):
        a = (a * (n-k+1))//k
        d[k, n-k] = d[n-k, k] = a
    return d

def binomial_coefficients_list(n):
    """Return a list of binomial coefficients as rows of the Pascal's
    triangle.
    """
    d = [1] * (n+1)
    a = 1
    for k in xrange(1, n//2+1):
        a = (a * (n-k+1))//k
        d[k] = d[n-k] = a
    return d

def multinomial_coefficients(m, n, _tuple=tuple, _zip=zip):
    """Return a dictionary containing pairs ``{(k1,k2,..,km) : C_kn}``
    where ``C_kn`` are multinomial coefficients such that
    ``n=k1+k2+..+km``.

    INPUT:
        m -- integer
        n -- integer
        _tuple, _zip -- hacks for speed; don't set these as a user.

    OUTPUT:
        dict

    EXAMPLES:
        >>> sorted(multinomial_coefficients(2,5).items())
        [((0, 5), 1), ((1, 4), 5), ((2, 3), 10), ((3, 2), 10), ((4, 1), 5), ((5, 0), 1)]

    Notice that these are the coefficients of ``(x+y)**5``:
         x**5 + 5*x**4*y + 10*x**3*y**2 + 10*x**2*y**3 + 5*x*y**4 + y**5
 
        >>> sorted(multinomial_coefficients(3,2).items())
        [((0, 0, 2), 1), ((0, 1, 1), 2), ((0, 2, 0), 1), ((1, 0, 1), 2), ((1, 1, 0), 2), ((2, 0, 0), 1)]


    ALGORITHM:
    The algorithm we implement for computing the multinomial coefficients
    is based on the following result:
    
       Consider a polynomial and its ``n``-th exponent::
       
         P(x) = sum_{i=0}^m p_i x^k
         P(x)^n = sum_{k=0}^{m n} a(n,k) x^k

       We compute the coefficients ``a(n,k)`` using the
       J.C.P. Miller Pure Recurrence [see D.E.Knuth, Seminumerical
       Algorithms, The art of Computer Programming v.2, Addison
       Wesley, Reading, 1981;]::
       
         a(n,k) = 1/(k p_0) sum_{i=1}^m p_i ((n+1)i-k) a(n,k-i),

       where ``a(n,0) = p_0^n``.
    """
    if m==0:
        return {}
    if m==2:
        return binomial_coefficients(n)
    symbols = [(0,)*i + (1,) + (0,)*(m-i-1) for i in range(m)]
    s0 = symbols[0]
    p0 = [_tuple([aa-bb for aa,bb in _zip(s,s0)]) for s in symbols]
    r = {_tuple([aa*n for aa in s0]):1}
    r_get = r.get
    r_update = r.update
    l = [0] * (n*(m-1)+1)
    l[0] = r.items()
    for k in xrange(1, n*(m-1)+1):
        d = {}
        d_get = d.get
        for i in xrange(1, min(m,k+1)):
            nn = (n+1)*i-k
            if not nn:
                continue
            t = p0[i]
            for t2, c2 in l[k-i]:
                tt = _tuple([aa+bb for aa,bb in _zip(t2,t)])
                cc = nn * c2
                dict_add_item(None, d, tt, cc)
        r1 = [(t, c//k) for (t, c) in d.iteritems()]
        l[k] = r1
        r_update(r1)
    return r

def reldiff(x,y):
    """abs(x-y)/((abs(x)+abs(y))/2)"""
    return abs(x-y)/((abs(x)+abs(y))/2)

def f2q(f, minerr=None):
    '''
    Find (almost) the best rational representation of a float that has
    finite precision.

    The algorithm is based on Stern-Brocot representation of
    irrational/rational numbers and its relation to continuants [1].

    References:
    [1] "Concrete Mathematics" by Graham, Knuth, and Patashnik. Addison-Wesley. 1989
    '''
    if mpmath.libmp.BACKEND=='gmpy':
        tmpz = type(mpmath.libmp.MPZ_ZERO)
        if minerr is None:
            r = mpmath.libmp.gmpy.f2q(f)
        else:
            r = mpmath.libmp.gmpy.f2q(f, minerr)
        if isinstance (r, tmpz):
            return int(r)
        n,d = int(r.numer ()), int(r.denom ())
        if d==1:
            return n
        return mpq((n,d))
    mpf = mpmath.mpf
    f = mpf(f)
    if f<0: return -f2q(-f,minerr)
    al = f                          # Initialize Eq.(6.142) [1]
    if not minerr:
        minerr = 1/mpf(2**al.context.prec) # min number for the given precision
    elif minerr<0:
        minerr = 1/mpf(2**-minerr)
    a = mpmath.floor(al)   # a = floor(al) # Initialize Eq.(6.142) [1]
    r1=[0,0,1]                           # r1[1:] is 1st row of Eq.(6.138) [1] 
    r2=[0,1,a]                           # r2[1:] is 2nd row of Eq.(6.138) [1]
    err = reldiff(f,a)
    while err>minerr:
        al = 1/(al-a)                    # Eq.(6.142) [1]
        a = mpmath.floor(al) # a = floor(al) # Eq.(6.142) [1]
        r1[0] = r1[1]                    # roll
        r1[1] = r1[2]                    # roll
        r1[2] = r1[1]*a+r1[0]            # use Eq.(6.127) [1]
        r2[0] = r2[1]                    # roll
        r2[1] = r2[2]                    # roll
        r2[2] = r2[1]*a+r2[0]            # use Eq.(6.127) [1]
        newerr = reldiff(f,r2[2]/r1[2])
        if err<=newerr:
            n, d = (int(r2[1]),int(r1[1]))
            if d==1: return n
            return mpq((n, d))
        err = newerr
    n, d = (int(r2[2]),int(r1[2]))
    if d==1: return n
    return mpq((n, d))


#from .combinatorics import multinomial_coefficients




from sympycore import CollectingField as Algebra

Symbol = Algebra.Symbol
Number = Algebra.Number
Add = Algebra.Add
Mul = Algebra.Mul
Pow = Algebra.Pow
Terms = Algebra.Terms
Factors = Algebra.Factors

def test_symbol():
    p = Symbol('p')
    s = Symbol('s')
    t = Symbol('t')
    assert s.matches(s)=={}
    assert s.matches(t)==None
    assert s.matches(t,{},([s,],[True,]))=={s:t}
    assert s.matches(t,{},([s,t],[True,True]))==None

def test_number():
    s = Symbol('s')
    n = Number(2)
    assert n.matches(2)=={}
    assert n.matches(3)==None
    assert n.matches(s)==None
    assert n.matches(s+2)==None

def test_wild():
    w = Symbol('w')
    s = Symbol('s')
    wargs = [w],[True]
    assert w.matches(Number(2),{},wargs)=={w:2}
    assert w.matches(s,{},wargs)=={w:s}
    assert w.matches(w,{},wargs)==None
    assert w.matches(s+2,{},wargs)=={w:s+2}
    assert w.matches(2*s,{},wargs)=={w:2*s}
    assert w.matches(s**2,{},wargs)=={w:s**2}

def test_symbol():
    s = Symbol('s')
    assert s.matches(s)=={}
    assert s.matches(2)==None
    assert s.matches(2+s)==None
    assert s.matches(2*s)==None
    assert s.matches(s**2)==None

def test_term():
    s = Symbol('s')
    p = 2*s
    assert p.matches(2*s)=={}
    assert p.matches(3*s)==None
    assert p.matches(s)==None
    assert p.matches(Number(2))==None
    assert p.matches(s**2)==None

def _test_wild_term():
    w = Symbol('w')
    p = 2*w
    s = Symbol('s')
    t = Symbol('t')
    wargs = {},([w],[True])
    assert p.matches(Number(1),*wargs)=={w:Number(1)/2}
    assert p.matches(Number(2),*wargs)=={w:1}
    assert p.matches(2*s,*wargs)=={w:s}
    assert p.matches(3*s,*wargs)=={w:s*Number(3)/2}
    assert p.matches(t*s,*wargs)=={w:t*s/2}
    assert p.matches(s**2,*wargs)=={w:s**2/2}
    m = p.matches(2*s+2,*wargs)
    assert m is not None and m[w]==(2*(s+1))/2
    assert p.matches(2*s+4,*wargs)=={w:(s+2)*2/2}
    assert p.matches(2*s+5,*wargs)=={w:(2*s+Number(5))/2}
    assert p.matches(2*s+t,*wargs)=={w:(2*s+t)/2}
    assert p.matches(2*s-2*t,*wargs)=={w:(s-t)*2/2}

def _test_wild_symbol_term():
    w = Symbol('w')
    s = Symbol('s')
    t = Symbol('t')
    p = s+w
    wargs = {},([w],[True])
    assert p.matches(s+2,*wargs)=={w:2}
    assert p.matches(t+2,*wargs)=={w:t+2-s}

def _test_wild_wild_term():
    w1 = Symbol('w1')
    w2 = Symbol('w2')

    p = w1 + 2*w2
    s = Symbol('s')
    t = Symbol('t')
    wargs = {},([w1,w2],[True,True])
    assert p.matches(Number(2),*wargs) in [{w2:0,w1:2},{w2:1,w1:0}]
    assert p.matches(2*s+t+2,*wargs) in [{w2:1+s,w1:t},{w1:2*s+t,w2:1},{w2:s,w1:t+2},
                                         {w1:2+2*s, w2:t/2}]

def _test_wild_factor():
    w = Symbol('w')
    p = w**2
    s = Symbol('s')
    t = Symbol('t')
    wargs = {},([w],[True])
    #assert p.matches(Number(2),*wargs)=={w:Number(2)**(Number(1)/2)}
    #assert p.matches(Number(4),*wargs)=={w:2}
    #assert p.matches(Number(16),*wargs)=={w:4}
    #assert p.matches(Number(9),*wargs)=={w:3}
    #assert p.matches(Number(8),*wargs)=={w:2*Number(2)**(Number(1)/2)}
    assert p.matches(s,*wargs)==None
    assert p.matches(s**2,*wargs)=={w:s}
    assert p.matches(s**3,*wargs)==None
    #assert p.matches(s**4,*wargs)=={w:s**2}
    assert p.matches(s+2,*wargs)==None
    assert p.matches(s*2,*wargs)==None
    assert p.matches(s**2*2,*wargs)==None
    #assert p.matches(s**2*4,*wargs)=={w:2*s}
    #assert p.matches(s**2*t**2,*wargs)=={w:s*t}
    #assert p.matches(4*s**2*t**2,*wargs)=={w:2*s*t}
    #assert p.matches(s**4*t**4,*wargs)=={w:(s*t)**2}
    #assert p.matches(s**2*t**4,*wargs)=={w:s*t**2}
    assert p.matches(s**2*t**3,*wargs)==None
    #assert p.matches(s**2*t**-4,*wargs)=={w:s*t**-2}


def _test_wild_symbol_factor():
    w = Symbol('w')
    s = Symbol('s')
    t = Symbol('t')
    p = s*w
    wargs = {},([w],[True])
    assert p.matches(Number(1),*wargs)=={w:1/s}
    assert p.matches(s,*wargs)=={w:1}
    assert p.matches(2+t,*wargs)=={w:(2+t)/s}

def test_symbol2():
    x = Symbol('x')
    a,b,c,p,q = map(Symbol, 'abcpq')
    e = x
    assert e.match(x) == {}
    assert e.match(a,a) == {a: x}

    e = Number(5)
    assert e.match(c,c) == {c: 5}
    assert e.match(e) == {}
    assert e.match(e+1) == None

def _test_add():
    x,y,a,b,c = map(Symbol, 'xyabc')
    p,q,r = map(Symbol, 'pqr')

    e = a+b
    assert e.match(p+b,p) == {p: a}
    assert e.match(p+a,p) == {p: b}

    e = 1+b
    assert e.match(p+b,p) == {p: 1}

    e = a+b+c
    assert e.match(a+p+c,p) == {p: b}
    assert e.match(b+p+c,p) == {p: a}

    e = a+b+c+x
    assert e.match(a+p+x+c,p) == {p: b}
    assert e.match(b+p+c+x,p) == {p: a}
    assert e.match(b) == None
    assert e.match(b+p,p) == {p: a+c+x}
    assert e.match(a+p+c,p) == {p: b+x}
    assert e.match(b+p+c,p) == {p: a+x}

    e = 4*x+5
    assert e.match(3*x+p,p) == {p: x+5}

    assert e.match(4*x+p,(p,lambda expr: not expr.args)) == {p: 5}
    assert e.match(p*x+5,(p,lambda expr: not expr.args)) == {p: 4}
    assert e.match(p*x+q,(p,lambda expr: not expr.args),(q,lambda expr: not expr.args)) == {p: 4, q: 5}

    e = 4*x+5*y+6
    assert e.match(p*x+q*y+r,(p,lambda expr: not expr.args),
                   (q,lambda expr: not expr.args),
                   (r,lambda expr: not expr.args)) == {p: 4, q: 5, r: 6}



#!/usr/bin/env python
#
# Created by Pearu Peterson in Febuary 2008
#

import os

from macros import preprocess
cwd = os.path.abspath(os.path.dirname(__file__))
targetfile_py = os.path.join(cwd,'..','sympycore','arithmetic','methods.py')

template = '''\
"""
Generated arithmetic methods for numbers.

This file is generated by the src/mk_numbers_methods.py script.
See http://sympycore.googlecode.com/ for more information.

DO NOT CHANGE THIS FILE DIRECTLY!!!
"""

from .numbers import mpqc, mpf, mpq, mpc, inttypes_set, realtypes_set, complextypes_set, numbertypes_set
'''

DIV_VALUE_VALUE = '''\
_p, _q = %(LHS)s, %(RHS)s
if not _q:
    raise ZeroDivisionError(repr(%(LHS)s) + " / " + repr(%(RHS)s))
@IF_CHECK_INT(T=type(_p))
    @IF_CHECK_INT(T=type(_q))
        @FRACTION_NORMALIZE(NUMER=_p; DENOM=_q; RNUMER=_rp; RDENOM=_rq; MOD=%(MOD)s)
        %(RESULT)s = _rp if _rq == 1 else mpq((_rp, _rq))
    else:
        %(RESULT)s = _p / _q
else:
    %(RESULT)s = _p / _q
'''

#=======================================================
# Constructor macros
#=======================================================

RETURN_FRACTION = '''\
return cls((%(NUMER)s, %(DENOM)s))
'''

FRACTION_NORMALIZE = '''\
%(RNUMER)s = _x = %(NUMER)s
%(RDENOM)s = _y = %(DENOM)s
while 1:
    _x, _y = _y, _x %(MOD)s _y
    if not _y:
        break
if _x != 1:
    %(RNUMER)s //= _x
    %(RDENOM)s //= _x
'''

RETURN_FRACTION2 = '''\
@FRACTION_NORMALIZE(NUMER=%(NUMER)s; DENOM=%(DENOM)s; RNUMER=_p; RDENOM=_q; MOD=%(MOD)s)
if _q == 1:
    return _p
@RETURN_FRACTION(NUMER=_p; DENOM=_q)
'''

RETURN_COMPLEX = '''\
%(TMP)s = new(cls)
%(TMP)s.real = %(REAL)s
%(TMP)s.imag = %(IMAG)s
return %(TMP)s
'''

RETURN_COMPLEX2 = '''\
%(TMP)s = %(IMAG)s
if not %(TMP)s:
    return %(REAL)s
@RETURN_COMPLEX(REAL=%(REAL)s; IMAG=%(TMP)s)
'''

#=======================================================
# ADD macros
#=======================================================

ADD_FRACTION_INT = '''\
p, q = %(LHS)s
@RETURN_FRACTION(NUMER=p+q*(%(RHS)s); DENOM=q)
'''
ADDOP_FRACTION_FRACTION = '''\
p, q = %(LHS)s
r, s = %(RHS)s
%(TMP)s = p*s %(SIGN)s q*r
if not %(TMP)s:
    return 0
@RETURN_FRACTION2(NUMER=%(TMP)s; DENOM=q*s; MOD=%(MOD)s)
'''
ADD_FRACTION_FRACTION = '''\
@ADDOP_FRACTION_FRACTION(LHS=%(LHS)s; RHS=%(RHS)s; SIGN=+; MOD=%%)
'''

ADD_COMPLEX_REAL = '@RETURN_COMPLEX(REAL=%(LHS)s.real + %(RHS)s; IMAG=%(LHS)s.imag)\n'
ADD_COMPLEX_COMPLEX = '@RETURN_COMPLEX2(REAL=%(LHS)s.real + %(RHS)s.real; IMAG=%(LHS)s.imag + %(RHS)s.imag)\n'

#=======================================================
# SUB macros
#=======================================================

SUB_FRACTION_INT = '''\
p, q = %(LHS)s
@RETURN_FRACTION(NUMER=p-q*(%(RHS)s); DENOM=q)
'''
SUB_INT_FRACTION = '''\
p, q = %(RHS)s
@RETURN_FRACTION(NUMER=q*(%(LHS)s) - p; DENOM=q)
'''
SUB_FRACTION_FRACTION = '''\
@ADDOP_FRACTION_FRACTION(LHS=%(LHS)s; RHS=%(RHS)s; SIGN=-; MOD=%%)
'''

SUB_COMPLEX_REAL = '@RETURN_COMPLEX(REAL=%(LHS)s.real - %(RHS)s; IMAG=%(LHS)s.imag)\n'
SUB_COMPLEX_COMPLEX = '@RETURN_COMPLEX2(REAL=%(LHS)s.real - %(RHS)s.real; IMAG=%(LHS)s.imag - %(RHS)s.imag)\n'
SUB_REAL_COMPLEX = '@RETURN_COMPLEX(REAL=%(LHS)s - %(RHS)s.real; IMAG=-%(RHS)s.imag)\n'

#=======================================================
# MUL macros
#=======================================================

MUL_FRACTION_INT = '''\
p, q = %(LHS)s
@RETURN_FRACTION2(NUMER=p*%(RHS)s; DENOM=q; MOD=%(MOD)s)
'''
MUL_FRACTION_FRACTION = '''\
p, q = %(LHS)s
r, s = %(RHS)s
@RETURN_FRACTION2(NUMER=p*r; DENOM=q*s; MOD=%(MOD)s)
'''

MUL_COMPLEX_REAL = '''\
if not %(RHS)s:
    return 0
@RETURN_COMPLEX(REAL=%(LHS)s.real*%(RHS)s; IMAG=%(LHS)s.imag*%(RHS)s)
'''
MUL_COMPLEX_COMPLEX = '''\
a, b = %(LHS)s.real, %(LHS)s.imag
c, d = %(RHS)s.real, %(RHS)s.imag
@RETURN_COMPLEX2(REAL=a*c-b*d; IMAG=b*c+a*d)
'''

#=======================================================
# DIV macros
#=======================================================

DIV_FRACTION_INT = '''\
p, q = %(LHS)s
@RETURN_FRACTION2(NUMER=p; DENOM=q*%(RHS)s; MOD=%(MOD)s)
'''
DIV_FRACTION_FRACTION = '''\
p, q = %(LHS)s
r, s = %(RHS)s
@RETURN_FRACTION2(NUMER=p*s; DENOM=q*r; MOD=%(MOD)s)
'''
DIV_INT_FRACTION = '''\
p, q = %(RHS)s
@RETURN_FRACTION2(NUMER=%(LHS)s*q; DENOM=p; MOD=%(MOD)s)
'''

DIV_COMPLEX_REAL = '''\
@DIV_VALUE_VALUE(LHS=%(LHS)s.real; RHS=%(RHS)s; RESULT=re; MOD=%(MOD)s)
@DIV_VALUE_VALUE(LHS=%(LHS)s.imag; RHS=%(RHS)s; RESULT=im; MOD=%(MOD)s)
@RETURN_COMPLEX(REAL=re; IMAG=im)
'''

DIV_COMPLEX_COMPLEX = '''\
a, b = %(LHS)s.real, %(LHS)s.imag
c, d = %(RHS)s.real, %(RHS)s.imag
mag = c*c + d*d
%(TMP)s = b*c-a*d
@DIV_VALUE_VALUE(LHS=a*c+b*d; RHS=mag; RESULT=re; MOD=%(MOD)s)
if not %(TMP)s:
    return re
@DIV_VALUE_VALUE(LHS=%(TMP)s; RHS=mag; RESULT=im; MOD=%(MOD)s)
@RETURN_COMPLEX(REAL=re; IMAG=im)
'''
DIV_REAL_COMPLEX = '''\
%(TMP)s = %(LHS)s
c, d = %(RHS)s.real, %(RHS)s.imag
mag = c*c + d*d
@DIV_VALUE_VALUE(LHS=-%(TMP)s*d; RHS=mag; RESULT=im; MOD=%(MOD)s)
@DIV_VALUE_VALUE(LHS= %(TMP)s*c; RHS=mag; RESULT=re; MOD=%(MOD)s)
@RETURN_COMPLEX(REAL=re; IMAG=im)
'''

#=======================================================
# POW macros
#=======================================================

POW_FRACTION_INT = '''\
%(TMP)s = %(RHS)s
p, q = %(LHS)s
if %(TMP)s > 0:
    @RETURN_FRACTION(NUMER=p**%(TMP)s; DENOM=q**%(TMP)s)
%(TMP)s = -%(TMP)s
if p > 0:
    @RETURN_FRACTION(NUMER=q**%(TMP)s; DENOM=p**%(TMP)s)
@RETURN_FRACTION(NUMER=(-q)**%(TMP)s; DENOM=(-p)**%(TMP)s)
'''

POW_COMPLEX_INT = '''\
a, b = %(LHS)s.real, %(LHS)s.imag
n = %(RHS)s
if not a:
    case = n %% 4
    if not case:
        return b**n
    elif case == 1:
        @RETURN_COMPLEX(REAL=0; IMAG=b**n)
    elif case == 2:
        return -(b**n)
    else:
        @RETURN_COMPLEX(REAL=0; IMAG=-b**n)
ta, tb = type(a), type(b)
m = 1
if ta is mpq:
    if tb is mpq:
        m = (a[1] * b[1]) ** n
        a, b = a[0]*b[1], a[1]*b[0]
    @ELIF_CHECK_INT(T=tb)
        m = a[1] ** n
        a, b = a[0], a[1]*b
elif tb is mpq:
    @IF_CHECK_INT(T=ta)
        m = b[1] ** n
        a, b = a*b[1], b[0]
c, d = 1, 0
while 1:
    if n & 1:
        c, d = a*c-b*d, b*c+a*d
        n -= 1
        if not n:
            break
    a, b = a*a-b*b, 2*a*b
    n //= 2
if m==1:
    @RETURN_COMPLEX2(REAL=c; IMAG=d)
# c,d,m are integers
@FRACTION_NORMALIZE(NUMER=c; DENOM=m; RNUMER=re_p; RDENOM=re_q; MOD=%(MOD)s)
re = re_p if re_q==1 else mpq((re_p, re_q))
if not d:
    return re
@FRACTION_NORMALIZE(NUMER=d; DENOM=m; RNUMER=im_p; RDENOM=im_q; MOD=%(MOD)s)
if im_q==1:
    im = im_p
else:
    im = mpq((im_p, im_q))
@RETURN_COMPLEX(REAL=re; IMAG=im)
'''


def main():
    f = open(targetfile_py, 'w')
    print >> f, template
    print >> f, preprocess('''

def fraction_add(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        @ADD_FRACTION_INT(LHS=self; RHS=other)
    elif t is cls:
        @ADD_FRACTION_FRACTION(LHS=self; RHS=other)
    @IF_CHECK_FLOAT(T=t)
        return self + mpf(other)
    return NotImplemented

def fraction_sub(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        @SUB_FRACTION_INT(LHS=self; RHS=other)
    elif t is cls:
        @SUB_FRACTION_FRACTION(LHS=self; RHS=other)
    @IF_CHECK_FLOAT(T=t)
        return self - mpf(other)
    return NotImplemented

def fraction_rsub(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        @SUB_INT_FRACTION(RHS=self; LHS=other)
    @IF_CHECK_FLOAT(T=t)
        return mpf(other) - self
    return NotImplemented

def fraction_mul(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    if t is cls:
        @MUL_FRACTION_FRACTION(LHS=self; RHS=other; MOD=%)
    @IF_CHECK_INT(T=t)
        @MUL_FRACTION_INT(LHS=self; RHS=other; MOD=%)
    @IF_CHECK_FLOAT(T=t)
        return self * mpf(other)
    return NotImplemented

def fraction_div(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        @DIV_FRACTION_INT(LHS=self; RHS=other; MOD=%)
    elif t is cls:
        @DIV_FRACTION_FRACTION(LHS=self; RHS=other; MOD=%)
    @IF_CHECK_FLOAT(T=t)
        return self / mpf(other)
    return NotImplemented

def fraction_rdiv(self, other, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        @DIV_INT_FRACTION(RHS=self; LHS=other; MOD=%)
    @IF_CHECK_FLOAT(T=t)
        return mpf(other) / self
    return NotImplemented

def fraction_pow(self, other, m=None, cls=mpq, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        if not other:
            return 1
        if other==1:
            return self
        @POW_FRACTION_INT(LHS=self; RHS=other)
    @IF_CHECK_FLOAT(T=t)
        return self ** mpf(other)
    return NotImplemented

def complex_add(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    @IF_CHECK_REAL(T=t)
        @ADD_COMPLEX_REAL(LHS=self; RHS=other)
    @IF_CHECK_COMPLEX(T=t)
        @ADD_COMPLEX_COMPLEX(LHS=self; RHS=other)
    return NotImplemented

def complex_sub(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    @IF_CHECK_REAL(T=t)
        @SUB_COMPLEX_REAL(LHS=self; RHS=other)
    @IF_CHECK_COMPLEX(T=t)
        @SUB_COMPLEX_COMPLEX(LHS=self; RHS=other)
    return NotImplemented

def complex_rsub(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set):
    t = type(other)
    @IF_CHECK_REAL(T=t)
        @SUB_REAL_COMPLEX(LHS=other; RHS=self)
    if t is complex:
        @SUB_COMPLEX_COMPLEX(LHS=other; RHS=self)
    return NotImplemented

def complex_mul(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    @IF_CHECK_COMPLEX(T=t)
        @MUL_COMPLEX_COMPLEX(LHS=self; RHS=other)
    @IF_CHECK_REAL(T=t)
        @MUL_COMPLEX_REAL(LHS=self; RHS=other)
    return NotImplemented

def complex_div(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set, complextypes_set=complextypes_set):
    t = type(other)
    @IF_CHECK_REAL(T=t)
        @DIV_COMPLEX_REAL(LHS=self; RHS=other; MOD=%)
    @IF_CHECK_COMPLEX(T=t)
        @DIV_COMPLEX_COMPLEX(LHS=self; RHS=other; MOD=%)
    return NotImplemented

def complex_rdiv(self, other, new=object.__new__, cls=mpqc, realtypes_set=realtypes_set):
    t = type(other)
    @IF_CHECK_REAL(T=t)
        @DIV_REAL_COMPLEX(LHS=other; RHS=self; MOD=%)
    if t is complex:
        @DIV_COMPLEX_COMPLEX(LHS=other; RHS=self; MOD=%)
    return NotImplemented

def complex_pow(self, other, m=None, new=object.__new__, cls=mpqc, inttypes_set=inttypes_set):
    t = type(other)
    @IF_CHECK_INT(T=t)
        if not other:
            return 1
        if other==1:
            return self
        if other==2:
            return self*self
        if other < 0:
            base, exponent = 1/self, -other
        else:
            base, exponent = self, other
        @POW_COMPLEX_INT(LHS=base; RHS=exponent; MOD=%)
    return NotImplemented
    ''', globals())

    for (n,s) in [('lt','<'), ('le','<='), ('gt','>'), ('ge','>=')]:
        print >> f, preprocess('''\
def fraction_%s(self, other, cls=mpq, inttypes_set=inttypes_set):
    p, q = self
    t = type(other)
    @IF_CHECK_INT(T=t)
        return (p %s q*other)
    elif t is cls:
        r, s = other
        return p*s %s q*r
    return NotImplemented
    ''' % (n, s, s), globals())

    f.close()


if __name__=='__main__':
    main()




from sympycore import *

def test_add():
    x = CommutativeRing('x')
    f = FunctionRing('f')
    g = FunctionRing('g')
    assert str(f+g) in ['f + g', 'g + f'], str(f+g)
    assert str((f+g)(x)) in ['f(x) + g(x)', 'g(x) + f(x)'],  str((f+g)(x))
    assert str((f+g)(x, evaluate=False)) in ['(f + g)(x)',
                                             '(g + f)(x)'], str((f+g)(x, evaluate=False))

def test_defined():
    x = Calculus('x')
    f = CalculusFunctionRing(heads.CALLABLE, Sin)
    g = CalculusFunctionRing('g')
    assert str(f(x))=='Sin(x)'
    assert str(f+g) in ['Sin + g','g + Sin'], str(f+g)
    assert str((f+g)(x)) in ['Sin(x) + g(x)','g(x) + Sin(x)'],str((f+g)(x))
    assert str((f+g)(x, evaluate=False)) in ['(Sin + g)(x)','(g + Sin)(x)'],str((f+g)(x, evaluate=False))


